from datetime import datetime

from dynatrace import Dynatrace
from dynatrace.environment_v2.monitored_entities import (
    Entity,
    EntityIcon,
    ToPosition,
    FromPosition,
    EntityType,
    EntityTypePropertyDto,
    MessageType,
    CustomDeviceCreation,
)
from dynatrace.environment_v2.schemas import ManagementZone
from dynatrace.environment_v2.custom_tags import METag

from dynatrace.pagination import PaginatedList
from dynatrace.utils import int64_to_datetime


def test_list(dt: Dynatrace):
    entities = dt.entities.list(
        'type("HOST")', fields="+fromRelationships,+toRelationships,+icon,+properties,+tags,+managementZones,+firstSeenTms,+lastSeenTms"
    )
    entities_list = list(entities)

    # type checks
    assert isinstance(entities, PaginatedList)
    assert all(isinstance(e, Entity) for e in entities)

    # value checks
    assert len(entities_list) == 1


def test_get(dt: Dynatrace):
    entity = dt.entities.get(
        "HOST-82F576674F19AC16",
        time_from=datetime.utcfromtimestamp(1618585701),
        time_to=datetime.utcfromtimestamp(1621177701),
        fields="+fromRelationships,+toRelationships,+icon,+properties,+tags,+managementZones,+firstSeenTms,+lastSeenTms",
    )

    # type checks
    assert isinstance(entity, Entity)
    assert isinstance(entity.entity_id, str)
    assert isinstance(entity.display_name, str)
    assert isinstance(entity.first_seen, (datetime, type(None)))
    assert isinstance(entity.last_seen, (datetime, type(None)))
    assert isinstance(entity.properties, dict)
    assert isinstance(entity.tags, list)
    assert all(isinstance(t, METag) for t in entity.tags)
    assert isinstance(entity.management_zones, list)
    assert all(isinstance(mz, ManagementZone) for mz in entity.management_zones)
    assert isinstance(entity.icon, EntityIcon)
    assert isinstance(entity.from_relationships, dict)
    assert isinstance(entity.to_relationships, dict)

    # value checks
    assert entity.entity_id == "HOST-82F576674F19AC16"
    assert entity.display_name == "arch-david"
    assert entity.first_seen == int64_to_datetime(1620821456242)
    assert entity.last_seen == int64_to_datetime(1621177614119)
    assert entity.properties["bitness"] == "64"
    assert len(entity.tags) == 1
    assert entity.tags[0].key == "citrix-prod"
    assert len(entity.management_zones) == 1
    assert entity.management_zones[0].id == "MANAGEMENT-ZONE-8E2ED64F4F19AC15"
    assert entity.icon.primary_icon_type == "linux"
    assert entity.from_relationships["isHostOfContainer"][0].id == "DOCKER_CONTAINER_GROUP_INSTANCE-8E2ED6F4E2AFDD89"
    assert entity.to_relationships["runsOn"][0].id == "PROCESS_GROUP-3AD9FB79C914520C"


def test_list_types(dt: Dynatrace):
    entity_types = dt.entities.list_types(page_size=3)
    entity_types_list = list(entity_types)

    # type checks
    assert isinstance(entity_types, PaginatedList)
    assert all(isinstance(et, EntityType) for et in entity_types)

    # value checks
    assert len(entity_types_list) == 2


def test_get_types(dt: Dynatrace):
    entity_type = dt.entities.get_type(entity_type="DISK")

    # type checks
    assert isinstance(entity_type, EntityType)
    assert isinstance(entity_type.type, str)
    assert isinstance(entity_type.dimension_key, str)
    assert isinstance(entity_type.entity_limit_exceeded, bool)
    assert isinstance(entity_type.properties, list)
    assert all(isinstance(p, EntityTypePropertyDto) for p in entity_type.properties)
    for prop in entity_type.properties:
        assert isinstance(prop.display_name, str)
        assert isinstance(prop.id, str)
        assert isinstance(prop.type, str)
    assert isinstance(entity_type.tags, str)
    assert isinstance(entity_type.management_zones, str)
    assert isinstance(entity_type.from_relationships, list)
    assert all(isinstance(fr, FromPosition) for fr in entity_type.from_relationships)
    for rel in entity_type.from_relationships:
        assert isinstance(rel.id, str)
        assert isinstance(rel.to_types, list)
        assert all(isinstance(tt, str) for tt in rel.to_types)
    assert isinstance(entity_type.to_relationships, list)
    assert all(isinstance(tr, ToPosition) for tr in entity_type.to_relationships)
    for rel in entity_type.to_relationships:
        assert isinstance(rel.id, str)
        assert isinstance(rel.from_types, list)
        assert all(isinstance(ft, str) for ft in rel.from_types)

    # value checks
    assert entity_type.type == "DISK"
    assert entity_type.display_name == "Disk"
    assert entity_type.dimension_key == "dt.entity.disk"
    assert entity_type.entity_limit_exceeded == False
    assert entity_type.properties[0].id == "awsNameTag"
    assert entity_type.properties[0].type == "String"
    assert entity_type.properties[0].display_name == "awsNameTag"
    assert entity_type.tags == "List"
    assert entity_type.management_zones == "List"
    assert entity_type.from_relationships[0].id == "isDiskOf"
    assert entity_type.from_relationships[0].to_types[0] == "HOST"
    assert entity_type.to_relationships[0].id == "isEbsVolumeOf"
    assert entity_type.to_relationships[0].from_types[0] == "EBS_VOLUME"


def test_create_custom_device(dt: Dynatrace):
    device = dt.entities.create_custom_device(
        custom_device_id="device-one",
        display_name="Test Device",
        group="group-one",
        device_type="IoT Beacon",
        ip_addresses=["10.20.0.0", "192.168.0.1"],
        listen_ports=[80, 8080],
        favicon_url="https://awesome-pics.img.com/icon",
        config_url="http://www.dashboard.com/device-one",
        dns_names=["testdevice.testnet.net"],
        properties={"this": "that", "lorem": "ipsum"},
        message_type=MessageType.CUSTOM_DEVICE,
    )

    # type checks
    assert isinstance(device, CustomDeviceCreation)
    assert isinstance(device.custom_device_id, str)
    assert isinstance(device.display_name, str)
    assert isinstance(device.group, str)
    assert isinstance(device.type, str)
    assert isinstance(device.ip_addresses, list)
    assert all(isinstance(ip, str) for ip in device.ip_addresses)
    assert isinstance(device.listen_ports, list)
    assert all(isinstance(port, int) for port in device.listen_ports)
    assert isinstance(device.favicon_url, str)
    assert isinstance(device.config_url, str)
    assert isinstance(device.dns_names, list)
    assert all(isinstance(dns, str) for dns in device.dns_names)
    assert isinstance(device.properties, dict)
    assert isinstance(device.message_type, MessageType)

    # value checks
    assert device.custom_device_id == "device-one"
    assert device.display_name == "Test Device"
    assert device.group == "group-one"
    assert device.type == "IoT Beacon"
    assert device.ip_addresses[0] == "10.20.0.0"
    assert device.listen_ports[0] == 80
    assert device.favicon_url == "https://awesome-pics.img.com/icon"
    assert device.config_url == "http://www.dashboard.com/device-one"
    assert device.dns_names[0] == "testdevice.testnet.net"
    assert device.properties["this"] == "that"
    assert device.message_type == MessageType.CUSTOM_DEVICE



"""
Copyright 2021 Dynatrace LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from enum import Enum
from typing import Optional, List, Union

from dynatrace.dynatrace_object import DynatraceObject
from dynatrace.http_client import HttpClient


class Unit(Enum):
    BIT = "Bit(bit)"
    BITPERHOUR = "BitPerHour(bit/h)"
    BITPERMINUTE = "BitPerMinute(bit/min)"
    BITPERSECOND = "BitPerSecond(bit/s)"
    BYTE = "Byte(B)"
    BYTEPERHOUR = "BytePerHour(B/h)"
    BYTEPERMINUTE = "BytePerMinute(B/min)"
    BYTEPERSECOND = "BytePerSecond(B/s)"
    CORES = "Cores"
    COUNT = "Count(count)"
    DAY = "Day(ds)"
    DECIBELMILLIWATT = "DecibelMilliWatt(dBm)"
    G = "G"
    GIBIBYTE = "GibiByte(GiB)"
    GIGABYTE = "GigaByte(GB)"
    HOUR = "Hour(hs)"
    KIBIBYTE = "KibiByte(KiB)"
    KIBIBYTEPERHOUR = "KibiBytePerHour(KiB/h)"
    KIBIBYTEPERMINUTE = "KibiBytePerMinute(KiB/min)"
    KIBIBYTEPERSECOND = "KibiBytePerSecond(KiB/s)"
    KILOBYTE = "KiloByte(kB)"
    KILOBYTEPERHOUR = "KiloBytePerHour(kB/h)"
    KILOBYTEPERMINUTE = "KiloBytePerMinute(kB/min)"
    KILOBYTEPERSECOND = "KiloBytePerSecond(kB/s)"
    M = "M"
    MEBIBYTE = "MebiByte(MiB)"
    MEBIBYTEPERHOUR = "MebiBytePerHour(MiB/h)"
    MEBIBYTEPERMINUTE = "MebiBytePerMinute(MiB/min)"
    MEBIBYTEPERSECOND = "MebiBytePerSecond(MiB/s)"
    MEGABYTE = "MegaByte(MB)"
    MEGABYTEPERHOUR = "MegaBytePerHour(MB/h)"
    MEGABYTEPERMINUTE = "MegaBytePerMinute(MB/min)"
    MEGABYTEPERSECOND = "MegaBytePerSecond(MB/s)"
    MICROSECOND = "MicroSecond(µs)"
    MILLISECOND = "MilliSecond(ms)"
    MILLISECONDPERMINUTE = "MilliSecondPerMinute(ms/min)"
    MINUTE = "Minute(mins)"
    MONTH = "Month(mos)"
    NOT_APPLICABLE = "N/A"
    NANOSECOND = "NanoSecond(ns)"
    NANOSECONDPERMINUTE = "NanoSecondPerMinute(ns/min)"
    PERHOUR = "PerHour(count/h)"
    PERMINUTE = "PerMinute(count/min)"
    PERSECOND = "PerSecond(count/s)"
    PERCENT = "Percent(%)"
    PIXEL = "Pixel(px)"
    PROMILLE = "Promille(‰)"
    RATIO = "Ratio"
    SECOND = "Second(s)"
    STATE = "State"
    UNSPECIFIED = "Unspecified"
    WEEK = "Week(ws)"
    YEAR = "Year(ys)"
    K = "k"
    M_CORES = "mCores"


class TimeSerieService:
    def __init__(self, http_client: HttpClient):
        self.__http_client = http_client

    def create_timeseries(
        self,
        metric_id: str,
        display_name: Optional[str],
        unit: Optional[Union[Unit, str]] = None,
        dimensions: Optional[List[str]] = None,
        technologies: Optional[List[str]] = None,
    ) -> "TimeseriesRegistrationMessage":

        unit: Unit = Unit(unit)
        return TimeseriesRegistrationMessage(
            self.__http_client,
            metric_id=metric_id,
            display_name=display_name,
            unit=unit.value,
            dimensions=dimensions,
            technologies=technologies,
        )


class TimeseriesRegistrationMessage(DynatraceObject):
    def __init__(
        self,
        http_client,
        metric_id: str,
        display_name: Optional[str],
        unit: Optional[str] = None,
        dimensions: Optional[List[str]] = None,
        technologies: Optional[List[str]] = None,
    ):
        self.metric_id = metric_id

        raw_element = {
            "displayName": display_name,
            "unit": unit,
            "dimensions": dimensions,
            "types": technologies,
        }
        super().__init__(http_client, None, raw_element)

    def put(self):
        return self._http_client.make_request(f"/api/v1/timeseries/{self.metric_id}", params=self._raw_element, method="PUT")



"""
Copyright 2021 Dynatrace LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from enum import Enum
from typing import List, Dict, Any, Optional

from dynatrace.dynatrace_object import DynatraceObject
from dynatrace.environment_v2.schemas import ConfigurationMetadata

# Schemas that don't belong to a specific API tag.


class UpdateWindowsConfig(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.windows: List[UpdateWindow] = [UpdateWindow(raw_element=uw) for uw in raw_element.get("windows", [])]

    def to_json(self) -> Optional[Dict[str, Any]]:
        if not self.windows:
            return None

        return {"windows": [w.to_json for w in self.windows]}


class UpdateWindow(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.id: str = raw_element.get("id", "")
        self.name: Optional[str] = raw_element.get("name", "")

    def to_json(self) -> Dict[str, Any]:
        return {
            "id": self.id,
            "name": self.name,
        }


class TechMonitoringList(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.technologies: List[Technology] = [Technology(raw_element=t) for t in raw_element.get("technologies", [])]
        self.metadata: ConfigurationMetadata = ConfigurationMetadata(raw_element=raw_element.get("metadata"))


class Technology(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.type: TechnologyType = TechnologyType(raw_element.get("type"))
        self.monitoring_enabled: bool = raw_element.get("monitoringEnabled", False)
        self.scope: Optional[SettingScope] = SettingScope(raw_element.get("scope")) if raw_element.get("scope") else None


class SettingScope(Enum):
    ENVIRONMENT = "ENVIRONMENT"
    HOST = "HOST"

    def __str__(self):
        return self.value


class TechnologyType(Enum):
    AIX_KERNEL_EXT = "AIX_KERNEL_EXT"
    APACHE = "APACHE"
    CIM_V2 = "CIM_V2"
    DOCKER = "DOCKER"
    DOCKER_WIN = "DOCKER_WIN"
    DOT_NET = "DOT_NET"
    DOT_NET_CORE = "DOT_NET_CORE"
    EXTENSIONS = "EXTENSIONS"
    EXTENSIONS_DS_GENERIC = "EXTENSIONS_DS_GENERIC"
    EXTENSIONS_STATSD = "EXTENSIONS_STATSD"
    GARDEN = "GARDEN"
    GO = "GO"
    GO_STATIC = "GO_STATIC"
    IBM_INTEGRATION_BUS = "IBM_INTEGRATION_BUS"
    IIS = "IIS"
    JAVA = "JAVA"
    LOG_ANALYTICS = "LOG_ANALYTICS"
    NETTRACER = "NETTRACER"
    NETWORK = "NETWORK"
    NGINX = "NGINX"
    NODE_JS = "NODE_JS"
    OPENTRACINGNATIVE = "OPENTRACINGNATIVE"
    PHP = "PHP"
    PHP_80_EA = "PHP_80_EA"
    PHP_CGI = "PHP_CGI"
    PHP_CLI = "PHP_CLI"
    PHP_WIN = "PHP_WIN"
    PROCESS = "PROCESS"
    RUBY = "RUBY"
    SDK = "SDK"
    VARNISH = "VARNISH"
    Z_OS = "Z_OS"

    def __str__(self):
        return self.value


class AutoUpdateSetting(Enum):
    ENABLED = "ENABLED"
    DISABLED = "DISABLED"
    INHERITED = "INHERITED"

    def __str__(self):
        return self.value


class EffectiveSetting(Enum):
    ENABLED = "ENABLED"
    DISABLED = "DISABLED"

    def __str__(self):
        return self.value


class TagContext(Enum):
    AWS = "AWS"
    AWS_GENERIC = "AWS_GENERIC"
    AZURE = "AZURE"
    CLOUD_FOUNDRY = "CLOUD_FOUNDRY"
    CONTEXTLESS = "CONTEXTLESS"
    ENVIRONMENT = "ENVIRONMENT"
    GOOGLE_CLOUD = "GOOGLE_CLOUD"
    KUBERNETES = "KUBERNETES"

    def __str__(self) -> str:
        return self.value


class StringComparisonOperator(Enum):
    BEGINS_WITH = "BEGINS_WITH"
    CONTAINS = "CONTAINS"
    CONTAINS_REGEX = "CONTAINS_REGEX"
    ENDS_WITH = "ENDS_WITH"
    EQUALS = "EQUALS"

    def __str__(self) -> str:
        return self.value



from datetime import datetime

from dynatrace import Dynatrace
from dynatrace.pagination import PaginatedList
from dynatrace.utils import int64_to_datetime
from dynatrace.environment_v2.events import Event, EventProperty, EventStatus, EventType, EventSeverity
from dynatrace.environment_v2.monitored_entities import EntityStub
from dynatrace.environment_v2.custom_tags import METag
from dynatrace.environment_v2.schemas import ManagementZone

EVENT_ID = "4578933396576863909_1631255744265"
EVENT_TYPE = "APPLICATION_OVERLOAD_PREVENTION"


def test_list(dt: Dynatrace):
    events = dt.events_v2.list(page_size=100, time_from=datetime.utcfromtimestamp(1599913748), time_to="1631258989895")

    # type checks
    assert isinstance(events, PaginatedList)
    assert all(isinstance(e, Event) for e in events)

    # value checks
    assert len(events) == 3


def test_get(dt: Dynatrace):
    event = dt.events_v2.get(EVENT_ID)

    # type checks
    assert isinstance(event, Event)
    assert isinstance(event.start_time, datetime)
    assert isinstance(event.end_time, datetime)
    assert isinstance(event.event_type, str)
    assert isinstance(event.title, str)
    assert isinstance(event.entity_id, EntityStub)
    assert isinstance(event.properties, list)
    assert all(isinstance(p, EventProperty) for p in event.properties)
    assert isinstance(event.status, EventStatus)
    assert isinstance(event.entity_tags, list)
    assert all(isinstance(tag, METag) for tag in event.entity_tags)
    assert isinstance(event.management_zones, list)
    assert all(isinstance(mz, ManagementZone) for mz in event.management_zones)
    assert isinstance(event.under_maintenance, bool)
    assert isinstance(event.suppress_alert, bool)
    assert isinstance(event.suppress_problem, bool)
    assert isinstance(event.frequent_event, bool)

    # value checks
    assert event.event_id == EVENT_ID
    assert event.start_time == int64_to_datetime(1631255744265)
    assert event.end_time == int64_to_datetime(1631255744265)
    assert event.event_type == "CUSTOM_DEPLOYMENT"
    assert event.title == "Deployment"
    assert event.properties[0].key == "dt.event.group_label"
    assert event.properties[0].value == "Deployment"
    assert event.status == EventStatus.CLOSED
    assert event.under_maintenance == False
    assert event.suppress_alert == False
    assert event.suppress_problem == False
    assert event.frequent_event == False


def test_list_types(dt: Dynatrace):
    event_types = dt.events_v2.list_types()

    # type checks
    assert isinstance(event_types, PaginatedList)
    assert all(isinstance(et, EventType) for et in event_types)

    # value checks
    assert len(event_types) == 5


def test_get_type(dt: Dynatrace):
    type_details = dt.events_v2.get_type(EVENT_TYPE)

    # type checks
    assert isinstance(type_details, EventType)
    assert isinstance(type_details.type, str)
    assert isinstance(type_details.display_name, str)
    assert isinstance(type_details.severity_level, EventSeverity)
    assert isinstance(type_details.description, str)

    # value checks
    assert type_details.type == EVENT_TYPE
    assert type_details.display_name == "Application overload prevention"
    assert type_details.severity_level == EventSeverity.INFO
    assert type_details.description == "Max user actions per minute exceeded"



from dynatrace import Dynatrace
from dynatrace.environment_v1.deployment import (
    InstallerMetaInfoDto,
    ConnectionInfo,
    AgentInstallerVersions,
    BoshReleaseAvailableVersions,
    ActiveGateInstallerVersions,
    ActiveGateConnectionInfo,
    BoshReleaseChecksum,
    LatestLambdaLayerNames,
)


VERSION = "1.215.159.20210428-145534"


def test_get_agent_installer_latest_metainfo(dt: Dynatrace):
    metainfo = dt.deployment.get_agent_installer_latest_metainfo(os_type="unix", installer_type="paas", flavor="musl", arch="x86", bitness="64")

    # type checks
    assert isinstance(metainfo, InstallerMetaInfoDto)
    assert isinstance(metainfo.latest_agent_version, str)

    # value checks
    assert metainfo.latest_agent_version == VERSION


def test_get_agent_installer_connection_info(dt: Dynatrace):
    info = dt.deployment.get_agent_installer_connection_info(version=VERSION)

    # type checks
    assert isinstance(info, ConnectionInfo)
    assert isinstance(info.tenant_uuid, str)
    assert isinstance(info.tenant_token, str)
    assert isinstance(info.communication_endpoints, list)
    assert all(isinstance(ce, str) for ce in info.communication_endpoints)
    assert isinstance(info.formatted_communication_endpoints, str)

    # value checks
    assert info.tenant_uuid == "abc12345"
    assert info.tenant_token == "4BcD3fGh1JkLmN0p"
    assert len(info.communication_endpoints) == 4
    assert info.communication_endpoints[0] == "https://host.docker.internal:9999/communication"
    assert info.formatted_communication_endpoints.startswith("{https://host.docker.internal:9999/communication")


def test_list_agent_installer_versions(dt: Dynatrace):
    versions = dt.deployment.list_agent_installer_versions(os_type="unix", installer_type="paas", flavor="musl", arch="x86")

    # type checks
    assert isinstance(versions, AgentInstallerVersions)
    assert isinstance(versions.available_versions, list)
    assert all(isinstance(av, str) for av in versions.available_versions)

    # value checks
    assert len(versions.available_versions) == 6
    assert versions.available_versions[0] == VERSION


def test_get_gateway_installer_connection_info(dt: Dynatrace):
    info = dt.deployment.get_gateway_installer_connection_info()

    # type checks
    assert isinstance(info, ActiveGateConnectionInfo)
    assert isinstance(info.tenant_token, str)
    assert isinstance(info.tenant_uuid, str)
    assert isinstance(info.communication_endpoints, str)

    # value checks
    assert info.tenant_uuid == "abc12345"
    assert info.tenant_token == "4BcD3fGh1JkLmN0p"
    assert info.communication_endpoints.startswith("https://sg-eu-west-1234.domain.com")


def test_list_gateway_installer_versions(dt: Dynatrace):
    versions = dt.deployment.list_gateway_installer_versions(os_type="unix")

    # type checks
    assert isinstance(versions, ActiveGateInstallerVersions)
    assert isinstance(versions.available_versions, list)
    assert all(isinstance(av, str) for av in versions.available_versions)

    # value checks
    assert len(versions.available_versions) == 6
    assert versions.available_versions[0] == VERSION


def test_list_boshrelease_agent_versions(dt: Dynatrace):
    versions = dt.deployment.list_boshrelease_agent_versions(os_type="unix")

    # type checks
    assert isinstance(versions, BoshReleaseAvailableVersions)
    assert isinstance(versions.available_versions, list)
    assert all(isinstance(av, str) for av in versions.available_versions)

    # value checks
    assert len(versions.available_versions) == 6
    assert versions.available_versions[0] == VERSION


def test_get_boshrelease_agent_checksum(dt: Dynatrace):
    checksum = dt.deployment.get_boshrelease_agent_checksum(os_type="unix", version=VERSION)

    # type checks
    assert isinstance(checksum, BoshReleaseChecksum)
    assert isinstance(checksum.sha_256, str)

    # value checks
    assert checksum.sha_256 == "8747793999922D34666A26F48C2061E598B164159015D12103A5D55A4F05225C"


def test_get_lambda_agent_versions(dt: Dynatrace):
    versions = dt.deployment.get_lambda_agent_versions()

    # type checks
    assert isinstance(versions, LatestLambdaLayerNames)
    assert isinstance(versions.java, str)
    assert isinstance(versions.nodejs, str)
    assert isinstance(versions.python, str)

    # value checks
    assert versions.java == "Dynatrace_OneAgent_1_221_103_20210713-172057"
    assert versions.python == "Dynatrace_OneAgent_1_221_3_20210624-164237"
    assert versions.nodejs == "Dynatrace_OneAgent_1_221_1_20210618-040655"



"""
Copyright 2021 Dynatrace LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from requests import Response
from typing import Dict, Any, Optional

from dynatrace.http_client import HttpClient
from dynatrace.dynatrace_object import DynatraceObject
from dynatrace.configuration_v1.schemas import UpdateWindowsConfig, AutoUpdateSetting, TechMonitoringList
from dynatrace.environment_v2.schemas import ConfigurationMetadata


class OneAgentEnvironmentWideConfigService:
    ENDPOINT = "/api/config/v1/hosts/autoupdate"

    def __init__(self, http_client: HttpClient):
        self.__http_client = http_client

    def get(self) -> "EnvironmentAutoUpdateConfig":
        """Gets the environment-wide configuration of OneAgents auto-update.

        :returns EnvironmentAutoUpdateConfig: the auto-update configuration for this environment
        """
        response = self.__http_client.make_request(path=self.ENDPOINT).json()
        return EnvironmentAutoUpdateConfig(raw_element=response, http_client=self.__http_client)

    def get_technologies(self) -> "TechMonitoringList":
        """Gets the global monitoring configuration of technologies.

        :returns TechMonitoringList: the technologies monitoring configuration for this environment
        """
        response = self.__http_client.make_request(path="/api/config/v1/technologies").json()
        return TechMonitoringList(raw_element=response)

    def put(self, config: "EnvironmentAutoUpdateConfig") -> "Response":
        """Updates the environment-wide configuration of OneAgents auto-update

        OneAgents are updated several minutes after the change of configuration.
        The process can take more time depending on number of OneAgents in the update queue.

        :param config: the updated global auto-update configuration object

        :returns Response: HTTP Response to the Request
        """
        return config.put()

    def is_valid(self, config: "EnvironmentAutoUpdateConfig") -> bool:
        """Validates the payload for the put function

        :param config: a global auto-update configuration object

        :returns bool: True if valid, false otherwise.
        """
        try:
            self.__http_client.make_request(path=f"{self.ENDPOINT}/validator", method="POST", params=config.to_json())
        except Exception as e:
            print(e.args)
            return False
        else:
            return True


class EnvironmentAutoUpdateConfig(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.metadata: ConfigurationMetadata = ConfigurationMetadata(raw_element=raw_element.get("metadata"))
        self.setting: AutoUpdateSetting = AutoUpdateSetting(raw_element.get("setting"))
        self.version: Optional[str] = raw_element.get("version")
        self.update_windows: UpdateWindowsConfig = UpdateWindowsConfig(raw_element=raw_element.get("updateWindows"))

    def to_json(self) -> Dict[str, Any]:
        return {
            "setting": str(self.setting),
            "version": self.version,
            "updateWindows": self.update_windows.to_json(),
        }

    def put(self) -> "Response":
        return self._http_client.make_request(path=OneAgentEnvironmentWideConfigService.ENDPOINT, method="PUT", params=self.to_json())



from datetime import datetime
from typing import List, Optional, Union, Dict, Any
from dynatrace import Dynatrace
from dynatrace.environment_v2.custom_tags import AddEntityTags, METag
from dynatrace.pagination import PaginatedList

import dynatrace.environment_v2.custom_tags as customtags

ENTITY_SELECTOR = "bilalhosts"


def test_list(dt: Dynatrace):
    _tags = dt.custom_tags.list(entity_selector=ENTITY_SELECTOR)

    # type checks
    assert isinstance(_tags, PaginatedList)
    assert len(list(_tags)) == 2
    assert all(isinstance(n, METag) for n in _tags)

    # value checks
    for t in _tags:
        assert t.key == "mainApp"
        assert t.string_representation == "mainApp"
        assert str(t.context) == "CONTEXTLESS"
        break


def test_post_no_value(dt: Dynatrace):
    tags = dt.custom_tags.post("entityId(CUSTOM_DEVICE-3B7788FE910B0F42)", [AddEntityTags("test-tag-no-value")])
    for tag in tags.applied_tags:
        assert str(tag.context) == "CONTEXTLESS"
        assert tag.key == "test-tag-no-value"
        assert tag.value is None
        assert tag.string_representation == "test-tag-no-value"


def test_post_value(dt: Dynatrace):
    tags = dt.custom_tags.post("entityId(CUSTOM_DEVICE-3B7788FE910B0F42)", [AddEntityTags("test-tag-value", "tag-value")])
    for tag in tags.applied_tags:
        assert str(tag.context) == "CONTEXTLESS"
        assert tag.key == "test-tag-value"
        assert tag.value == "tag-value"
        assert tag.string_representation == "test-tag-value:tag-value"


def test_delete(dt: Dynatrace):
    deleted_tags = dt.custom_tags.delete("test-tag-value", "entityId(CUSTOM_DEVICE-3B7788FE910B0F42)", delete_all_with_key=True)
    assert deleted_tags.matched_entities_count == 1



"""
Copyright 2021 Dynatrace LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from enum import Enum
from requests import Response
from datetime import datetime
from typing import Optional, Union, Dict, Any, List

from dynatrace.http_client import HttpClient
from dynatrace.dynatrace_object import DynatraceObject
from dynatrace.environment_v2.schemas import ManagementZone
from dynatrace.environment_v2.metrics import Unit
from dynatrace.environment_v2.monitored_entities import EntityStub
from dynatrace.environment_v2.custom_tags import METag
from dynatrace.configuration_v1.alerting_profiles import AlertingProfileStub
from dynatrace.pagination import PaginatedList
from dynatrace.utils import int64_to_datetime


class ProblemService:
    ENDPOINT = "/api/v2/problems"

    def __init__(self, http_client: HttpClient):
        self.__http_client = http_client

    def list(
        self,
        problem_selector: Optional[str] = None,
        entity_selector: Optional[str] = None,
        fields: Optional[str] = None,
        time_from: Optional[Union[datetime, str]] = None,
        time_to: Optional[Union[datetime, str]] = None,
        sort: Optional[str] = None,
        page_size: Optional[int] = None,
    ) -> PaginatedList["Problem"]:
        """Gets a list of Problems based on the given criteria.

        :return: a list of Problems along with their details.
        """
        params = {
            "problemSelector": problem_selector,
            "entitySelector": entity_selector,
            "fields": fields,
            "from": time_from,
            "to": time_to,
            "sort": sort,
            "pageSize": page_size,
        }
        return PaginatedList(target_class=Problem, http_client=self.__http_client, target_url=self.ENDPOINT, target_params=params, list_item="problems")

    def get(self, problem_id: str, fields: Optional[str] = None) -> "Problem":
        """Gets a Problem by specifying its id.

        :param problem_id: the ID of the Problem
        :return: a Problem along with its details
        """
        params = {"fields": fields}
        response = self.__http_client.make_request(path=f"{self.ENDPOINT}/{problem_id}", params=params).json()
        return Problem(raw_element=response)

    def close(self, problem_id: str, message: str) -> "ProblemCloseResult":
        """Closes an open Problem leaving a closing message as comment

        :param message: message to leave as closing comment
        :param problem_id: the ID of the Problem
        :return: details of the closed problem result. Blank details if the problem was already closed.
        """
        params = {"message": message}
        response = self.__http_client.make_request(path=f"{self.ENDPOINT}/{problem_id}/close", method="POST", params=params).json()
        return ProblemCloseResult(raw_element=response)

    def list_comments(self, problem_id: str, page_size: Optional[int] = 10) -> PaginatedList["Comment"]:
        """Gets a list of comments belonging to a given Problem.

        :param problem_id: the ID of the Problem
        :param page_size: the number of comments in a single response payload
        :return: a list of the Problem's comments
        """
        params = {"pageSize": page_size}
        return PaginatedList(
            target_class=Comment, http_client=self.__http_client, target_url=f"api/v2/{problem_id}/comments", target_params=params, list_item="comments"
        )

    def get_comment(self, problem_id: str, comment_id: str) -> "Comment":
        """Gets a specific Comment from a specific Problem

        :param problem_id: the ID of the Problem
        :param comment_id: the ID of the Comment
        :return: a Comment along with its details
        """
        response = self.__http_client.make_request(path=f"{self.ENDPOINT}/{problem_id}/comments/{comment_id}").json()
        return Comment(raw_element=response)

    def add_comment(self, problem_id: str, message: str, context: Optional[str] = None) -> Response:
        """Adds a new Comment on the specified Problem.

        :param problem_id: the ID of the Problem
        :param message: the text body of the Comment to add
        :param context: the optional context to attach to the Comment
        :return: HTTP Response
        """
        params = {"message": message, "context": context}
        return self.__http_client.make_request(path=f"{self.ENDPOINT}/{problem_id}/comments", params=params, method="POST")

    def update_comment(self, problem_id: str, comment_id: str, message: str, context: Optional[str] = None) -> Response:
        """Updates the specified Comment on a Problem.

        :param problem_id: the ID of the Problem
        :param comment_id: the ID of the Comment
        :param message: the new text body of the Comment to update
        :param context: the optional context to update on the Comment
        :return: HTTP Response
        """
        params = {"message": message, "context": context}
        return self.__http_client.make_request(path=f"{self.ENDPOINT}/{problem_id}/comments/{comment_id}", params=params, method="PUT")

    def delete_comment(self, problem_id: str, comment_id: str) -> Response:
        """Deletes the specified Comment from a Problem.

        :param problem_id: the ID of the Problem
        :param comment_id: the ID of the Comment
        :return: HTTP Response
        """
        return self.__http_client.make_request(path=f"{self.ENDPOINT}/{problem_id}/comments/{comment_id}", method="DELETE")


class Problem(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        # required
        self.display_id: str = raw_element.get("displayId")
        self.problem_id: str = raw_element.get("problemId")
        self.title: str = raw_element.get("title")
        self.status: str = Status(raw_element.get("status"))
        self.severity_level: str = SeverityLevel(raw_element.get("severityLevel"))
        self.impact_level: str = ImpactLevel(raw_element.get("impactLevel"))
        self.start_time: datetime = int64_to_datetime(raw_element.get("startTime"))
        self.end_time: datetime = int64_to_datetime(raw_element.get("endTime")) if raw_element.get("endTime") != -1 else None

        # optional
        self.management_zones: Optional[List[ManagementZone]] = [ManagementZone(raw_element=m) for m in raw_element.get("managementZones", [])]
        self.affected_entities: Optional[List[EntityStub]] = [EntityStub(raw_element=e) for e in raw_element.get("affectedEntities", [])]
        self.recent_comments: Optional[CommentList] = CommentList(raw_element=raw_element.get("recentComments"))
        self.impacted_entities: Optional[List[EntityStub]] = [EntityStub(raw_element=e) for e in raw_element.get("impactedEntities", [])]
        self.linked_problem_info: Optional[LinkedProblem] = LinkedProblem(raw_element=raw_element.get("linkedProblemInfo"))
        self.root_cause_entity: Optional[EntityStub] = EntityStub(raw_element=raw_element["rootCauseEntity"]) if raw_element.get("rootCauseEntity") else None
        self.problem_filters: Optional[List[AlertingProfileStub]] = [AlertingProfileStub(raw_element=a) for a in raw_element.get("problemFilters", [])]
        self.evidence_details: Optional[EvidenceDetails] = EvidenceDetails(raw_element=raw_element.get("evidenceDetails"))
        self.impact_analysis: Optional[ImpactAnalysis] = ImpactAnalysis(raw_element=raw_element.get("impactAnalysis"))
        self.entity_tags: Optional[List[METag]] = [METag(raw_element=t) for t in raw_element.get("entityTags", [])]


class ProblemCloseResult(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.problem_id: str = raw_element.get("problemId")
        self.closing: bool = raw_element.get("closing")
        self.close_timestamp: datetime = int64_to_datetime(raw_element.get("closeTimestamp"))
        self.comment: Optional[Comment] = Comment(raw_element=raw_element.get("comment"))


class LinkedProblem(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.display_id: str = raw_element.get("displayId")
        self.problem_id: str = raw_element.get("problemId")


class CommentList(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.total_count: int = raw_element.get("totalCount")
        self.comments: Optional[List[Comment]] = [Comment(raw_element=c) for c in raw_element.get("comments", [])]


class Comment(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.created_at: datetime = int64_to_datetime(raw_element.get("createdAtTimestamp"))
        self.author: Optional[str] = raw_element.get("authorName")
        self.context: Optional[str] = raw_element.get("context")
        self.id: Optional[str] = raw_element.get("id")
        self.content: Optional[str] = raw_element.get("content")


class EvidenceDetails(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        raw_details = raw_element.get("details", [])
        self.total_count: int = raw_element.get("totalCount")
        self.details: Optional[List[Evidence]] = (
            [EventEvidence(raw_element=e) for e in raw_details if e.get("evidenceType") == EvidenceType.EVENT.value]
            + [MetricEvidence(raw_element=e) for e in raw_details if e.get("evidenceType") == EvidenceType.METRIC.value]
            + [TransactionalEvidence(raw_element=e) for e in raw_details if e.get("evidenceType") == EvidenceType.TRANSACTIONAL.value]
            + [MaintenanceWindowEvidence(raw_element=e) for e in raw_details if e.get("evidenceType") == EvidenceType.MAINTENANCE_WINDOW.value]
            + [AvailabilityEvidence(raw_element=e) for e in raw_details if e.get("evidenceType") == EvidenceType.AVAILABILITY.value]
        )


class Evidence(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.evidence_type: EvidenceType = EvidenceType(raw_element.get("evidenceType"))
        self.display_name: str = raw_element.get("displayName")
        self.entity: EntityStub = EntityStub(raw_element=raw_element.get("entity"))
        self.root_cause_relevant: bool = raw_element.get("rootCauseRelevant")
        self.start_time: datetime = int64_to_datetime(raw_element.get("startTime"))
        self.grouping_entity: Optional[EntityStub] = EntityStub(raw_element=raw_element.get("groupingEntity")) if raw_element.get("groupingEntity") else None


class EventEvidence(Evidence):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        super()._create_from_raw_data(raw_element)
        self.event_id: Optional[str] = raw_element.get("eventId")
        self.event_type: Optional[str] = raw_element.get("eventType")


class TransactionalEvidence(Evidence):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        super()._create_from_raw_data(raw_element)
        self.value_before_change_point: Optional[float] = raw_element.get("valueBeforeChangePoint")
        self.value_after_change_point: Optional[float] = raw_element.get("valueAfterChangePoint")
        self.end_time: Optional[datetime] = int64_to_datetime(raw_element.get("endTime"))
        self.unit: Optional[Unit] = Unit(raw_element.get("unit"))


class MetricEvidence(Evidence):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        super()._create_from_raw_data(raw_element)
        self.value_before_change_point: Optional[float] = raw_element.get("valueBeforeChangePoint")
        self.value_after_change_point: Optional[float] = raw_element.get("valueAfterChangePoint")
        self.end_time: Optional[datetime] = int64_to_datetime(raw_element.get("endTime"))
        self.unit: Optional[Unit] = Unit(raw_element.get("unit"))
        self.metric_id: Optional[str] = raw_element.get("metricId")


class AvailabilityEvidence(Evidence):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        super()._create_from_raw_data(raw_element)
        self.end_time: Optional[datetime] = int64_to_datetime(raw_element.get("endTime"))


class MaintenanceWindowEvidence(Evidence):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        super()._create_from_raw_data(raw_element)
        self.maintenance_window_id: Optional[str] = raw_element.get("maintenanceWindowConfigId")
        self.end_time: Optional[datetime] = int64_to_datetime(raw_element.get("endTime"))


class ImpactAnalysis(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.impacts: List[Impact] = [Impact(raw_element=i) for i in raw_element.get("impacts", [])]


class Impact(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.impact_type: ImpactType = ImpactType(raw_element.get("impactType"))
        self.impacted_entity: EntityStub = EntityStub(raw_element=raw_element.get("impactedEntity"))
        self.estimated_affected_users: int = raw_element.get("estimatedAffectedUsers")


class SeverityLevel(Enum):
    AVAILABILITY = "AVAILABILITY"
    CUSTOM_ALERT = "CUSTOM_ALERT"
    ERROR = "ERROR"
    MONITORING_UNAVAILABLE = "MONITORING_UNAVAILABLE"
    PERFORMANCE = "PERFORMANCE"
    RESOURCE_CONTENTION = "RESOURCE_CONTENTION"


class ImpactLevel(Enum):
    APPLICATION = "APPLICATION"
    ENVIRONMENT = "ENVIRONMENT"
    INFRASTRUCTURE = "INFRASTRUCTURE"
    SERVICES = "SERVICES"


class Status(Enum):
    OPEN = "OPEN"
    CLOSED = "CLOSED"


class EvidenceType(Enum):
    EVENT = "EVENT"
    METRIC = "METRIC"
    TRANSACTIONAL = "TRANSACTIONAL"
    MAINTENANCE_WINDOW = "MAINTENANCE_WINDOW"
    AVAILABILITY = "AVAILABILITY_EVIDENCE"


class ImpactType(Enum):
    SERVICE = "SERVICE"
    APPLICATION = "APPLICATION"
    MOBILE = "MOBILE"
    CUSTOM_APPLICATION = "CUSTOM_APPLICATION"



from dynatrace import Dynatrace
from datetime import datetime

from dynatrace.environment_v2.audit_logs import AuditLogEntry, EventType, Category, UserType
from dynatrace.pagination import PaginatedList


def test_list(dt: Dynatrace):
    audit_logs = dt.audit_logs.list()
    assert isinstance(audit_logs, PaginatedList)

    audit_logs_list = list(audit_logs)
    assert len(audit_logs_list) == 6

    first = audit_logs_list[0]
    assert isinstance(first, AuditLogEntry)
    assert first.log_id == "162100314800090003"
    assert first.event_type == EventType("DELETE")
    assert first.category == Category("CONFIG")
    assert first.entity_id == "builtin:alerting.profile (tenant): d89472d3-f9f4-420d-9398-768bb3351e85: test"
    assert first.environment_id == "eaa50379"
    assert first.user == "Dynatrace support user #649982176"
    assert first.user_type == UserType("USER_NAME")
    assert first.user_origin == "webui (xxx.xxx.xxx.xxx)"
    assert first.timestamp == datetime.utcfromtimestamp(1621003148800 / 1000)
    assert first.success


def test_get(dt: Dynatrace):
    audit_log = dt.audit_logs.get("162100314800090003")
    assert isinstance(audit_log, AuditLogEntry)
    assert audit_log.log_id == "162100314800090003"
    assert audit_log.event_type == EventType("DELETE")
    assert audit_log.category == Category("CONFIG")
    assert audit_log.entity_id == "builtin:alerting.profile (tenant): d89472d3-f9f4-420d-9398-768bb3351e85: test"
    assert audit_log.environment_id == "eaa50379"
    assert audit_log.user == "Dynatrace support user #649982176"
    assert audit_log.user_type == UserType("USER_NAME")
    assert audit_log.user_origin == "webui (xxx.xxx.xxx.xxx)"
    assert audit_log.timestamp == datetime.utcfromtimestamp(1621003148800 / 1000)
    assert audit_log.success



from dynatrace import Dynatrace
from dynatrace.configuration_v1.alerting_profiles import (
    AlertingProfileStub,
    AlertingProfile,
    AlertingProfileSeverityRule,
    SeverityLevel,
    AlertingProfileTagFilter,
    TagFilterIncludeMode,
    AlertingPredefinedEventFilter,
    AlertingEventTypeFilter,
    AlertingCustomEventFilter,
    AlertingCustomTextFilter,
    AlertingPredefinedEvent,
)
from dynatrace.configuration_v1.schemas import ConfigurationMetadata, StringComparisonOperator
from dynatrace.environment_v2.monitored_entities import EntityShortRepresentation
from dynatrace.environment_v2.custom_tags import METag
from dynatrace.pagination import PaginatedList

ID = "b1f379d9-98b4-4efe-be38-0289609c9295"
NAME = "deployment_change_autoremediation"


def test_list(dt: Dynatrace):
    alert_profiles = dt.alerting_profiles.list()
    assert isinstance(alert_profiles, PaginatedList)

    list_alert_profiles = list(alert_profiles)
    assert len(list_alert_profiles) == 6

    first = list_alert_profiles[0]
    assert isinstance(first, AlertingProfileStub)

    assert first.id == ID
    assert first.name == NAME


def test_get_full_configuration(dt: Dynatrace):
    alert_profiles = dt.alerting_profiles.list()
    list_alert_profiles = list(alert_profiles)
    first = list_alert_profiles[0]

    full = first.get_full_configuration()
    assert isinstance(full, AlertingProfile)
    assert full.id == ID
    assert full.display_name == NAME
    assert isinstance(full.rules, list)
    assert isinstance(full.rules[0], AlertingProfileSeverityRule)


def test_get(dt: Dynatrace):
    ap = dt.alerting_profiles.get(profile_id=ID)

    # type checks
    assert isinstance(ap, AlertingProfile)
    assert isinstance(ap.metadata, ConfigurationMetadata)
    assert isinstance(ap.id, str)
    assert isinstance(ap.display_name, str)
    assert isinstance(ap.rules, list)
    assert all(isinstance(rule, AlertingProfileSeverityRule) for rule in ap.rules)
    for rule in ap.rules:
        assert isinstance(rule.severity_level, SeverityLevel)
        assert isinstance(rule.tag_filter, AlertingProfileTagFilter)
        assert isinstance(rule.tag_filter.include_mode, TagFilterIncludeMode)
        assert isinstance(rule.tag_filter.tag_filters, list)
        for tf in rule.tag_filter.tag_filters:
            assert isinstance(tf, METag)
        assert isinstance(rule.delay_in_minutes, int)
    assert isinstance(ap.management_zone_id, str)
    assert isinstance(ap.event_type_filters, list)
    first_event = ap.event_type_filters[0]
    assert isinstance(first_event, AlertingEventTypeFilter)
    assert isinstance(first_event.custom_event_filter, AlertingCustomEventFilter)
    assert isinstance(first_event.custom_event_filter.custom_title_filter, AlertingCustomTextFilter)
    assert isinstance(first_event.custom_event_filter.custom_title_filter.enabled, bool)
    assert isinstance(first_event.custom_event_filter.custom_title_filter.value, str)
    assert isinstance(first_event.custom_event_filter.custom_title_filter.operator, StringComparisonOperator)
    assert isinstance(first_event.custom_event_filter.custom_title_filter.negate, bool)
    assert isinstance(first_event.custom_event_filter.custom_title_filter.case_insensitive, bool)
    second_event = ap.event_type_filters[1]
    assert isinstance(second_event, AlertingEventTypeFilter)
    assert isinstance(second_event.predefined_event_filter, AlertingPredefinedEventFilter)
    assert isinstance(second_event.predefined_event_filter.event_type, AlertingPredefinedEvent)
    assert isinstance(second_event.predefined_event_filter.negate, bool)

    # value checks
    assert ap.id == ID
    assert ap.display_name == NAME
    rule = ap.rules[0]
    assert rule.severity_level == SeverityLevel.PERFORMANCE
    assert rule.tag_filter.include_mode == TagFilterIncludeMode.INCLUDE_ANY
    assert rule.delay_in_minutes == 25
    assert ap.management_zone_id == "-6238974133282121422"
    custom_event_filter = ap.event_type_filters[0].custom_event_filter.custom_title_filter
    predef_event_filter = ap.event_type_filters[1].predefined_event_filter
    assert custom_event_filter.enabled == True
    assert custom_event_filter.value == "ERROR"
    assert custom_event_filter.operator == StringComparisonOperator.CONTAINS
    assert custom_event_filter.negate == False
    assert custom_event_filter.case_insensitive == False
    assert predef_event_filter.event_type == AlertingPredefinedEvent.OSI_HIGH_CPU
    assert predef_event_filter.negate == False


def test_post(dt: Dynatrace):
    response = dt.alerting_profiles.post(
        AlertingProfile(
            raw_element={
                "id": ID,
                "displayName": NAME,
                "description": "new_management_zone",
                "rules": [
                    {
                        "severityLevel": "PERFORMANCE",
                        "tagFilter": {"includeMode": "INCLUDE_ANY", "tagFilters": [{"context": "CONTEXTLESS", "key": "Application", "value": "Custom"}]},
                        "delayInMinutes": 25,
                    }
                ],
            }
        )
    )

    # type checks
    assert isinstance(response, EntityShortRepresentation)
    # value checks
    assert response.id == ID
    assert response.name == NAME



"""
Copyright 2021 Dynatrace LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from datetime import datetime
from dynatrace.dynatrace_object import DynatraceObject
from typing import List, Optional, Union, Dict, Any

from dynatrace.http_client import HttpClient
from dynatrace.pagination import PaginatedList


class NetworkZoneService:
    ENDPOINT = "/api/v2/networkZones"
    ENDPOINT_GLOBALSETTINGS = "/api/v2/networkZoneSettings"

    def __init__(self, http_client: HttpClient):
        self.__http_client = http_client

    def list(self) -> PaginatedList["NetworkZone"]:
        """Lists all network zones. No params

        :return: a list of Network Zones with details
        """
        return PaginatedList(NetworkZone, self.__http_client, target_url=self.ENDPOINT, list_item="networkZones")

    def get(self, networkzone_id: str):
        """Gets parameters of specified network zone

        :param networkzone_id: the ID of the network zone
        :return: a Network Zone + details
        """
        response = self.__http_client.make_request(f"{self.ENDPOINT}/{networkzone_id}").json()
        return NetworkZone(raw_element=response)

    def update(self, networkzone_id: str, alternate_zones: Optional[List[str]] = None, description: Optional[str] = None):
        """Updates an existing network zone or creates a new one

        :param networkzone_id: the ID of the network zone, if none exists, will create
        :param alternate_zones: optional list of text body of alternative network zones
        :param description: optional text body for short description of network zone
        :return: HTTP response
        """
        params = {"alternativeZones": alternate_zones, "description": description}
        return self.__http_client.make_request(path=f"{self.ENDPOINT}/{networkzone_id}", params=params, method="PUT")

    def delete(self, networkzone_id: str):
        """Deletes the specified network zone

        :param networkzone_id: the ID of the network zone
        :return: HTTP response
        """
        return self.__http_client.make_request(path=f"{self.ENDPOINT}/{networkzone_id}", method="DELETE")

    def getGlobalConfig(self):
        """Gets the global configuration of network zones. No params
        :return: Network Zone Global Settings object
        """
        response = self.__http_client.make_request(path=self.ENDPOINT_GLOBALSETTINGS).json()
        return NetworkZoneSettings(raw_element=response)

    def updateGlobalConfig(self, configuration: bool):
        """Updates the global configuration of network zones.

        :param configuration: boolean setting to enable/disable NZs
        :return: HTTP response
        """
        params = {"networkZonesEnabled": configuration}
        return self.__http_client.make_request(path=self.ENDPOINT_GLOBALSETTINGS, method="PUT", params=params)


class NetworkZone(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.id: str = raw_element.get("id")
        self.description: str = raw_element.get("description")
        self.alternative_zones: List[str] = raw_element.get("alternativeZones")
        self.num_oneagents_using: int = raw_element.get("numOfOneAgentsUsing")
        self.num_oneagents_configured: int = raw_element.get("numOfConfiguredOneAgents")
        self.num_oneagents_from_other_zones: int = raw_element.get("numOfOneAgentsFromOtherZones")
        self.num_configured_activegates: int = raw_element.get("numOfConfiguredActiveGates")


class NetworkZoneSettings(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, bool]):
        self.network_zones_enabled: bool = raw_element.get("networkZonesEnabled")



from datetime import datetime
from dynatrace import Dynatrace
from dynatrace.pagination import PaginatedList

import dynatrace.environment_v2.networkzones as nz

NETWORKZONE_ID = "default"


def test_list(dt: Dynatrace):
    network_zones = dt.network_zones.list()

    # type checks
    assert isinstance(network_zones, PaginatedList)
    assert len(list(network_zones)) == 2
    assert all(isinstance(n, nz.NetworkZone) for n in network_zones)


def test_get(dt: Dynatrace):
    network_zone = dt.network_zones.get(networkzone_id=NETWORKZONE_ID)

    # type checks
    assert isinstance(network_zone, nz.NetworkZone)
    assert isinstance(network_zone.alternative_zones, list)

    # value checks
    assert (
        network_zone.description
        == "The default network zone. This is the network zone for OneAgents or ActiveGates that do not have any network zone configured."
    )
    assert network_zone.id == "default"
    assert network_zone.num_configured_activegates == 0
    assert network_zone.num_oneagents_configured == 141
    assert network_zone.num_oneagents_using == 141
    assert network_zone.num_oneagents_from_other_zones == 0



from dynatrace import Dynatrace

from dynatrace.environment_v2.metrics import MetricDescriptor, Unit, AggregationType, Transformation, ValueType, MetricSeriesCollection
from dynatrace.pagination import PaginatedList
from dynatrace.utils import int64_to_datetime


def test_list(dt: Dynatrace):
    metrics = dt.metrics.list()
    assert isinstance(metrics, PaginatedList)
    for metric in metrics:
        assert isinstance(metric, MetricDescriptor)
        assert metric.metric_id == "builtin:apps.other.apdex.osAndGeo"
        assert metric.display_name == "Apdex (by OS, geolocation) [mobile, custom]"
        assert metric.description == ""
        assert metric.unit == Unit.NOTAPPLICABLE
        break


def test_list_fields(dt: Dynatrace):
    metrics = dt.metrics.list(
        fields="+tags,+dduBillable,+created,+lastWritten,+aggregationTypes,+defaultAggregation,+dimensionDefinitions,+transformations,+entityType"
    )
    assert isinstance(metrics, PaginatedList)

    for metric in metrics:
        assert isinstance(metric, MetricDescriptor)
        assert metric.metric_id == "builtin:apps.other.apdex.osAndGeo"
        assert metric.display_name == "Apdex (by OS, geolocation) [mobile, custom]"
        assert metric.description == ""
        assert metric.unit == Unit.NOTAPPLICABLE
        assert not metric.ddu_billable
        assert metric.created is None
        assert metric.last_written == int64_to_datetime(1620514220905)
        assert metric.entity_type == ["CUSTOM_APPLICATION", "MOBILE_APPLICATION"]
        assert metric.aggregation_types == [AggregationType.AUTO, AggregationType.AUTO.VALUE]
        assert Transformation.FOLD in metric.transformations
        assert metric.default_aggregation.type == "value"
        assert len(metric.dimension_definitions) == 3
        assert metric.dimension_definitions[0].key == "dt.entity.device_application"
        assert metric.dimension_definitions[0].name == "Application"
        assert metric.dimension_definitions[0].index == 0
        assert metric.dimension_definitions[0].type == "ENTITY"
        break


def test_list_params(dt: Dynatrace):

    written_since = int64_to_datetime(1621029621)

    metrics = dt.metrics.list(
        metric_selector="builtin:host.*",
        written_since=written_since,
        metadata_selector='unit("Percent")',
        fields="+tags,+dduBillable,+created,+lastWritten,+aggregationTypes,+defaultAggregation,+dimensionDefinitions,+transformations,+entityType",
    )

    for metric in metrics:
        assert metric.metric_id == "builtin:host.cpu.idle"
        assert metric.display_name == "CPU idle"
        assert metric.description == ""
        assert metric.unit == Unit.PERCENT
        assert not metric.ddu_billable
        assert metric.last_written == int64_to_datetime(1621030025348)
        assert metric.entity_type == ["HOST"]
        assert metric.default_aggregation.type == "avg"
        assert metric.tags == []
        assert metric.dimension_definitions[0].key == "dt.entity.host"
        break


def test_get(dt: Dynatrace):

    metric = dt.metrics.get("builtin:host.cpu.idle")

    assert metric.metric_id == "builtin:host.cpu.idle"
    assert metric.display_name == "CPU idle"
    assert metric.description == ""
    assert metric.unit == Unit.PERCENT
    assert not metric.ddu_billable
    assert metric.last_written == int64_to_datetime(1621030565348)
    assert metric.entity_type == ["HOST"]
    assert metric.default_aggregation.type == "avg"
    assert metric.tags == []
    assert metric.dimension_definitions[0].key == "dt.entity.host"
    assert metric.metric_value_type.type == ValueType.SCORE


def test_query(dt: Dynatrace):
    time_from = int64_to_datetime(1621020000000)
    time_to = int64_to_datetime(1621025000000)

    results = dt.metrics.query("builtin:host.cpu.idle", time_from=time_from, time_to=time_to)
    assert isinstance(results, PaginatedList)

    first = list(results)[0]

    assert isinstance(first, MetricSeriesCollection)
    assert first.metric_id == "builtin:host.cpu.idle"
    assert len(first.data) == 1

    first_data = first.data[0]
    assert first_data.dimension_map == {"dt.entity.host": "HOST-82F576674F19AC16"}
    assert first_data.dimensions == ["HOST-82F576674F19AC16"]
    assert len(first_data.timestamps) == 84
    assert len(first_data.timestamps) == len(first_data.values)
    assert first_data.timestamps[0] == int64_to_datetime(1621020060000)
    assert first_data.values[0] == 89.91581217447917


def test_ingest(dt: Dynatrace):
    ingest = dt.metrics.ingest(["a 1", "b 2"])
    assert isinstance(ingest, dict)
    assert ingest["linesOk"] == 1
    assert ingest["linesInvalid"] == 0
    assert ingest["error"] is None



from typing import List
from dynatrace import Dynatrace
from dynatrace.configuration_v1.auto_tags import (
    AutoTag,
    AutoTagRule,
    AutoTagRuleType,
    AutoTagShortRepresentation,
    ComparisonBasic,
    ComparisonBasicType,
    ConditionKey,
    ConditionKeyAttribute,
    EntityRuleEngineCondition,
)
from dynatrace.environment_v2.schemas import ConfigurationMetadata
from dynatrace.pagination import PaginatedList


def test_list(dt: Dynatrace):
    tags = dt.auto_tags.list()
    assert isinstance(tags, PaginatedList)

    for tag in tags:
        assert isinstance(tag, AutoTagShortRepresentation)
        assert tag.id == "403e033b-7324-4bfe-bef1-b3f367de42f2"
        assert tag.name == "frontend"
        break


def test_get(dt: Dynatrace):
    tags = dt.auto_tags.list()
    for tag in tags:
        print(tag.name)
        full_tag = tag.get_full_configuration()
        assert isinstance(full_tag, AutoTag)
        assert isinstance(full_tag.metadata, ConfigurationMetadata)
        assert full_tag.metadata.cluster_version == "1.214.112.20210409-064503"
        assert full_tag.id == "403e033b-7324-4bfe-bef1-b3f367de42f2"
        assert full_tag.name == "frontend"
        assert isinstance(full_tag.rules, List)
        for rule in full_tag.rules:
            print(rule)
            assert isinstance(rule, AutoTagRule)
            assert rule.type == AutoTagRuleType.SERVICE
            assert rule.enabled == True
            assert rule.value_format == ""
            assert rule.propagation_types == []
            assert isinstance(rule.conditions, List)
            for condition in rule.conditions:
                assert isinstance(condition, EntityRuleEngineCondition)
                assert isinstance(condition.key, ConditionKey)
                assert condition.key.attribute == ConditionKeyAttribute.PROCESS_GROUP_NAME
                assert isinstance(condition.comparison_info, ComparisonBasic)
                assert condition.comparison_info.type == ComparisonBasicType.STRING
                assert condition.comparison_info.operator == "CONTAINS"
                assert condition.comparison_info.value == "frontend"
                assert condition.comparison_info.negate == False
                assert condition.comparison_info.case_sensitive == False
                break
            break
        break



from dynatrace import Dynatrace
from dynatrace.environment_v2.extensions import MinimalExtension
from dynatrace.pagination import PaginatedList

import dynatrace.environment_v2.extensions as extensions_v2


def test_list(dt: Dynatrace):
    extensions = list(dt.extensions_v2.list())

    assert len(extensions) == 2

    for extension in extensions:
        assert isinstance(extension, MinimalExtension)
        assert extension.extension_name == "com.dynatrace.extension.snmp-generic"
        assert extension.version == "0.2.5"
        break


def test_list_name(dt: Dynatrace):
    extensions = list(dt.extensions_v2.list(name="custom"))

    assert len(extensions) == 1

    for extension in extensions:
        assert isinstance(extension, MinimalExtension)
        assert extension.extension_name == "custom:dynatrace.cisco.asa"
        assert extension.version == "0.1.0"
        break


def test_list_versions(dt: Dynatrace):
    extensions = list(dt.extensions_v2.list_versions("com.dynatrace.extension.snmp-generic"))

    assert len(extensions) == 1

    for extension in extensions:
        assert isinstance(extension, MinimalExtension)
        assert extension.extension_name == "com.dynatrace.extension.snmp-generic"
        assert extension.version == "0.2.5"
        break


def test_get(dt: Dynatrace):
    extension = dt.extensions_v2.get("com.dynatrace.extension.snmp-generic", "0.2.5")

    # type checks
    assert isinstance(extension, extensions_v2.Extension)
    assert isinstance(extension.author, extensions_v2.AuthorDTO)
    assert isinstance(extension.data_sources, list)
    assert isinstance(extension.feature_sets, list)
    assert isinstance(extension.variables, list)

    # value checks
    assert extension.extension_name == "com.dynatrace.extension.snmp-generic"
    assert extension.author.name == "Dynatrace"
    assert extension.version == "0.2.5"
    assert extension.variables[0] == "ext.activationtag"


def test_get_active_extension_version(dt: Dynatrace):
    environemnt_config = dt.extensions_v2.get_environment_config("ibmmq")

    # type checks
    assert isinstance(environemnt_config, extensions_v2.ExtensionEnvironmentConfigurationVersion)

    # value checks
    assert environemnt_config.version == "1.2.3"



from dynatrace import Dynatrace


def test_start_rotation(dt: Dynatrace):
    token_config = dt.tenant_tokens.start_rotation()
    assert token_config.active.value == "hRCnr6Yd3BFrtxaF"
    assert token_config.old.value == "prv0bYw93v8sU9b1"


def test_cancel_rotation(dt: Dynatrace):
    token_config = dt.tenant_tokens.cancel_rotation()
    assert token_config.active.value == "prv0bYw93v8sU9b1"
    assert token_config.old.value is None


def test_finish_rotation(dt: Dynatrace):
    token_config = dt.tenant_tokens.finish_rotation()
    assert token_config.active.value == "prv0bYw93v8sU9b1"
    assert token_config.old.value is None



"""
Copyright 2021 Dynatrace LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from datetime import datetime, timedelta, timezone
from collections.abc import MutableSequence
from typing import Optional, List, Dict, Tuple


from dynatrace.dynatrace_object import DynatraceObject
from dynatrace.http_client import HttpClient


class CustomDeviceService:
    def __init__(self, http_client: HttpClient):
        self.__http_client = http_client

    def create(
        self,
        device_id: str,
        display_name: Optional[str] = None,
        group: Optional[str] = None,
        ip_addresses: Optional[List[str]] = None,
        listen_ports: Optional[List[int]] = None,
        technology: Optional[str] = None,
        favicon: Optional[str] = None,
        config_url: Optional[str] = None,
        properties: Optional[Dict[str, str]] = None,
        tags: Optional[List[str]] = None,
        series: Optional[List] = None,
        host_names: Optional[List[str]] = None,
    ) -> "CustomDevicePushMessage":
        return CustomDevicePushMessage(
            self.__http_client,
            device_id=device_id,
            display_name=display_name,
            group=group,
            ip_addresses=ip_addresses,
            listen_ports=listen_ports,
            technology=technology,
            favicon=favicon,
            config_url=config_url,
            properties=properties,
            tags=tags,
            series=series,
            host_names=host_names,
        )


class Series(MutableSequence):
    def __init__(self, *args):
        self.list: List["EntityTimeseriesData"] = []
        self.extend(list(args))

    def append(self, time_series: "EntityTimeseriesData") -> None:
        for element in self.list:
            if time_series.timeseries_id == element.timeseries_id and time_series.dimensions == element.dimensions:
                element.data_points.extend(time_series.data_points)
                return
        self.list.append(time_series)

    def __len__(self):
        return len(self.list)

    def __getitem__(self, i):
        return self.list[i]

    def __delitem__(self, i):
        del self.list[i]

    def __setitem__(self, i, v):
        self.list[i] = v

    def insert(self, i, v):
        self.list.insert(i, v)

    def __str__(self):
        return str(self.list)


class CustomDevicePushMessage(DynatraceObject):
    def __init__(
        self,
        http_client,
        device_id: str,
        display_name: Optional[str] = None,
        group: Optional[str] = None,
        ip_addresses: Optional[List[str]] = None,
        listen_ports: Optional[List[int]] = None,
        technology: Optional[str] = None,
        favicon: Optional[str] = None,
        config_url: Optional[str] = None,
        properties: Optional[Dict[str, str]] = None,
        tags: Optional[List[str]] = None,
        series: Optional[Series] = None,
        host_names: Optional[List[str]] = None,
    ):
        self.device_id = device_id
        self.display_name: Optional[str] = display_name
        self.group: Optional[str] = group
        self.ip_addresses: Optional[List[str]] = ip_addresses
        self.listen_ports: Optional[List[int]] = listen_ports
        self.technology: Optional[str] = technology
        self.favicon: Optional[str] = favicon
        self.config_url: Optional[str] = config_url
        self.properties: Optional[Dict[str, str]] = properties
        self.tags: Optional[List[str]] = tags
        self.__series: Series = series
        if self.__series is None:
            self.__series: Series = Series()
        self.host_names: Optional[List[str]] = host_names

        raw_element = {
            "displayName": self.display_name,
            "group": self.group,
            "ipAddresses": self.ip_addresses,
            "listenPorts": self.listen_ports,
            "type": self.technology,
            "favicon": self.favicon,
            "configUrl": self.config_url,
            "properties": self.properties,
            "tags": self.tags,
            "series": [s._raw_element for s in self.__series] if self.__series else None,
            "hostNames": self.host_names,
        }
        super().__init__(http_client, None, raw_element)

    @property
    def series(self) -> Series:
        return self.__series

    @series.setter
    def series(self, series: Series):
        self.__series = series
        self._raw_element["series"] = [s._raw_element for s in self.__series]

    def post(self, only_valid_data_points=False):
        try:
            response = self._http_client.make_request(f"/api/v1/entity/infrastructure/custom/{self.device_id}", params=self._raw_element, method="POST")
            return response
        except Exception as e:
            if only_valid_data_points and ("configuration.Creation timestamp is:" in f"{e}" or "Data point timestamp is too far in the past" in f"{e}"):
                if "configuration.Creation timestamp" in f"{e}":
                    max_timestamp = int(f"{e}".split("configuration.Creation timestamp is:")[1].split('"')[0].strip())
                    max_timestamp = datetime.fromtimestamp(max_timestamp / 1000, tz=timezone.utc)
                else:
                    max_timestamp = datetime.now(tz=timezone.utc) - timedelta(minutes=59)
                self._http_client.log.warning(f"Some data points were invalid, removing data points older than {max_timestamp}")
                for s in self.series:
                    s.data_points = [d for d in s.data_points if d.timestamp.replace(tzinfo=max_timestamp.tzinfo) >= max_timestamp]
                self._raw_element["series"] = [s._raw_element for s in self.series]
                return self.post()
            else:
                raise e

    def absolute(self, key: str, value: float, timestamp: Optional[datetime] = None, dimensions: Optional[Dict[str, str]] = None):
        data_point = DataPoint(value, timestamp)
        self.series.append(EntityTimeseriesData(self._http_client, key, [data_point], dimensions))
        self.series = self.series  # Ugly as hell hack because of setter, and I don't want to subclass list


class EntityTimeseriesData(DynatraceObject):
    def __init__(self, http_client, timeseries_id: str, data_points: List["DataPoint"], dimensions: Optional[Dict[str, str]] = None):
        self.timeseries_id: str = timeseries_id
        self.dimensions = dimensions
        self.__data_points: List["DataPoint"] = data_points
        raw_element = {
            "timeseriesId": timeseries_id,
            "dimensions": dimensions,
            "dataPoints": [[int(data_point.timestamp.timestamp() * 1000), data_point.value] for data_point in data_points],
        }
        super().__init__(http_client, None, raw_element)

    def __repr__(self):
        return f"Series(id={self.timeseries_id}, data_points={self.data_points})"

    @property
    def data_points(self) -> List["DataPoint"]:
        return self.__data_points

    @data_points.setter
    def data_points(self, data_points: List["DataPoint"]):
        self.__data_points = data_points
        self._raw_element["dataPoints"] = [[int(data_point.timestamp.timestamp() * 1000), data_point.value] for data_point in self.__data_points]


class DataPoint:
    def __init__(self, value: float, timestamp: Optional[datetime] = None):
        self.timestamp = timestamp
        if self.timestamp is None:
            self.timestamp = datetime.now()
        self.value = value

    def __repr__(self):
        return f"[{self.timestamp}, {self.value}]"



"""
Copyright 2021 Dynatrace LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from datetime import datetime
from typing import Optional, List, Dict

from requests import Response

from dynatrace.dynatrace_object import DynatraceObject
from dynatrace.http_client import HttpClient

EVENT_TYPE_AVAILABILITY_EVENT = "AVAILABILITY_EVENT"
EVENT_TYPE_CUSTOM_ALERT = "CUSTOM_ALERT"
EVENT_TYPE_CUSTOM_ANNOTATION = "CUSTOM_ANNOTATION"
EVENT_TYPE_CUSTOM_CONFIGURATION = "CUSTOM_CONFIGURATION"
EVENT_TYPE_CUSTOM_DEPLOYMENT = "CUSTOM_DEPLOYMENT"
EVENT_TYPE_CUSTOM_INFO = "CUSTOM_INFO"
EVENT_TYPE_ERROR_EVENT = "ERROR_EVENT"
EVENT_TYPE_MARKED_FOR_TERMINATION = "MARKED_FOR_TERMINATION"
EVENT_TYPE_PERFORMANCE_EVENT = "PERFORMANCE_EVENT"
EVENT_TYPE_RESOURCE_CONTENTION = "RESOURCE_CONTENTION"


class EventService:
    def __init__(self, http_client: HttpClient):
        self.__http_client = http_client

    def create_event(
        self,
        event_type: str,
        entity_id: str,
        source: str,
        start: Optional[datetime] = None,
        end: Optional[datetime] = None,
        timeout_minutes: Optional[int] = None,
        annotation_type: Optional[str] = None,
        annotation_description: Optional[str] = None,
        description: Optional[str] = None,
        title: Optional[str] = None,
        custom_properties: Optional[Dict[str, str]] = None,
        allow_davis_merge: Optional[bool] = None,
    ) -> Response:

        attach_rules = PushEventAttachRules(entity_ids=[entity_id], tag_rule=None)
        return EventCreation(
            self.__http_client,
            event_type=event_type,
            attach_rules=attach_rules,
            source=source,
            start=start,
            end=end,
            timeout_minutes=timeout_minutes,
            annotation_type=annotation_type,
            annotation_description=annotation_description,
            description=description,
            title=title,
            custom_properties=custom_properties,
            allow_davis_merge=allow_davis_merge,
        ).post()


class EventCreation(DynatraceObject):
    def __init__(
        self,
        http_client,
        event_type: str,
        attach_rules: "PushEventAttachRules",
        source: str,
        start: Optional[datetime] = None,
        end: Optional[datetime] = None,
        timeout_minutes: Optional[int] = None,
        annotation_type: Optional[str] = None,
        annotation_description: Optional[str] = None,
        description: Optional[str] = None,
        title: Optional[str] = None,
        custom_properties: Optional[str] = None,
        allow_davis_merge: Optional[bool] = None,
    ):

        raw_element = {
            "eventType": event_type,
            "start": int(start.timestamp()) * 1000 if start else None,
            "end": int(end.timestamp()) * 1000 if end else None,
            "timeoutMinutes": timeout_minutes,
            "source": source,
            "annotationType": annotation_type,
            "annotationDescription": annotation_description,
            "attachRules": attach_rules._raw_element,
            "description": description,
            "title": title,
            "customProperties": custom_properties,
            "allowDavisMerge": allow_davis_merge,
        }

        super().__init__(http_client, None, raw_element)

    def post(self):
        return self._http_client.make_request(f"/api/v1/events", params=self._raw_element, method="POST")


class PushEventAttachRules:
    def __init__(self, entity_ids: Optional[List[str]], tag_rule: Optional[List["TagMatchRule"]]):

        self._raw_element = {
            "entityIds": entity_ids,
            "tagRule": [t._raw_element for t in tag_rule] if tag_rule else None,
        }


class TagMatchRule:
    def __init__(self, me_types: List[str], tags: List[str]):
        self._raw_element = {
            "meTypes": me_types,
            "tags": tags,
        }


"""
type EventStoreResult struct {
	StoredEventIds       []int    `json:"storedEventIds,omitempty"`
	StoredIds            []string `json:"storedIds,omitempty"`
	StoredCorrelationIds []string `json:"storedCorrelationIds,omitempty"`
}
"""



"""
Copyright 2021 Dynatrace LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from typing import Dict, Any, Union, List

from requests import Response

from dynatrace.http_client import HttpClient


class LogService:
    # TODO - Add search and get
    ENDPOINT = "/api/v2/logs"

    def __init__(self, http_client: HttpClient):
        self.__http_client = http_client

    def ingest(self, payload: Union[Dict[str, Any], List[Dict[str, Any]]]) -> Response:
        """
        Ingests logs into the Dynatrace log store.
        :param payload: A list of log entries or a single log entry, which are JSON objects (dictionaries)
        :return: The HTTP Response
        """
        headers = {"Content-Type": "application/json; charset=utf-8"}
        return self.__http_client.make_request(f"{self.ENDPOINT}/ingest", params=payload, method="POST", headers=headers)



"""
Copyright 2021 Dynatrace LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from datetime import datetime
from enum import Enum
from requests import Response
from typing import List, Optional, Union, Dict, Any

from dynatrace.environment_v2.schemas import ManagementZone
from dynatrace.environment_v2.custom_tags import METag
from dynatrace.http_client import HttpClient
from dynatrace.dynatrace_object import DynatraceObject
from dynatrace.pagination import PaginatedList
from dynatrace.utils import int64_to_datetime, timestamp_to_string


class EntityService:
    ENDPOINT_ENTITIES = "/api/v2/entities"
    ENDPOINT_TYPES = "/api/v2/entityTypes"

    def __init__(self, http_client: HttpClient):
        self.__http_client = http_client

    def list(
        self,
        entity_selector: str,
        time_from: Optional[Union[datetime, str]] = None,
        time_to: Optional[Union[datetime, str]] = None,
        fields: Optional[str] = None,
        sort: Optional[str] = None,
        page_size: Optional[int] = None,
    ) -> PaginatedList["Entity"]:
        """Gets the information about monitored entities.

        Lists entities observed within the specified timeframe along with their properties.
        When you query entities of the SERVICE_METHOD type, only the following requests are returned:
            - Key requests
            - Top X requests that are used for baselining
            - Requests that have caused a problem

        :param page_size: The amount of entities per page. If not set, 50 is used.
        :param entity_selector: Defines the scope of the query. Only entities matching the specified criteria are included into response.
                                You must set one of these criteria:
                                    - Entity type: type("TYPE")
                                    - Dynatrace entity ID: entityId("id"). Several IDs can be separated by a comma (entityId("id-1","id-2")).
                                The length of the string is limited to 10,000 characters.
        :param time_from: The start of the requested timeframe. If not set, the relative timeframe of three days is used (now-3d).
        :param time_to: The end of the requested timeframe. If not set, the current timestamp is used.
        :param fields: Defines the list of entity properties included in the response. The ID and the name of an entity are always included to the response.
        :param sort: Defines the ordering of the entities returned. Currently ordering is only available for the display name (for example sort=name or sort =+name for ascending, sort=-name for descending)

        :return: A list of monitored entities along with their properties.
        """
        params = {
            "pageSize": page_size,
            "entitySelector": entity_selector,
            "from": timestamp_to_string(time_from),
            "to": timestamp_to_string(time_to),
            "fields": fields,
            "sort": sort,
        }
        return PaginatedList(Entity, self.__http_client, self.ENDPOINT_ENTITIES, target_params=params, list_item="entities")

    def get(
        self, entity_id: str, time_from: Optional[Union[datetime, str]] = None, time_to: Optional[Union[datetime, str]] = None, fields: Optional[str] = None
    ) -> "Entity":
        """Gets the properties of the specified monitored entity.

        :param entity_id: The ID of the required entity.
        :param time_from: The start of the requested timeframe. If not set, the relative timeframe of three days is used (now-3d).
        :param time_to: The end of the requested timeframe. If not set, the current timestamp is used.
        :param fields: Defines the list of entity properties included in the response. The ID and the name of an entity are always included to the response.

        :returns Entity: the monitored entity requested
        """
        params = {"from": timestamp_to_string(time_from), "to": timestamp_to_string(time_to), "fields": fields}
        response = self.__http_client.make_request(f"{self.ENDPOINT_ENTITIES}/{entity_id}", params=params).json()
        return Entity(raw_element=response)

    def post_custom_device(self, device: "CustomDeviceCreation") -> "Response":
        """Creates or updates a custom device.

        If the Custom Device ID matches an existing device, the respective parameters will be updated.

        :returns Response: HTTP Response for the request
        """
        return device.post()

    def create_custom_device(
        self,
        custom_device_id: str,
        display_name: str,
        group: Optional[str] = None,
        ip_addresses: Optional[List[str]] = None,
        listen_ports: Optional[List[int]] = None,
        device_type: Optional[str] = None,
        favicon_url: Optional[str] = None,
        config_url: Optional[str] = None,
        properties: Optional[Dict[str, str]] = None,
        dns_names: Optional[List[str]] = None,
        message_type: Optional["MessageType"] = None,
    ) -> "CustomDeviceCreation":
        """Creates a Custom Device object from scratch.

        :param custom_device_id: The internal ID of the custom device. Max. length 512.
        :param display_name: The name of the custom device to be displayed in the user interface.
        :param group: User defined group ID of entity.
        :param ip_addresses: The list of IP addresses that belong to the custom device.
        :param listen_ports: The list of ports the custom devices listens to.
        :param device_type: The technology type definition of the custom device.
        :param favicon_url: The icon to be displayed for your custom component within Smartscape. Provide the full URL of the icon file.
        :param config_url: The URL of a configuration web page for the custom device, such as a login page for a firewall or router.
        :param properties: Key-value pair properties that will be shown beneath the infographics of your custom device.
        :param dns_names: The list of DNS names related to the custom device.
        :param message_type: Use the Enum for possible values

        :returns CustomDeviceCreation: The resulting Custom Device. This can now be used in POST calls.
        """
        raw_device = {
            "customDeviceId": custom_device_id,
            "displayName": display_name,
            "group": group,
            "ipAddresses": ip_addresses,
            "listenPorts": listen_ports,
            "type": device_type,
            "faviconUrl": favicon_url,
            "configUrl": config_url,
            "properties": properties,
            "dnsNames": dns_names,
            "messageType": message_type,
        }
        return CustomDeviceCreation(raw_element=raw_device, http_client=self.__http_client)

    def list_types(self, page_size: Optional[int] = 50) -> PaginatedList["EntityType"]:
        """
        Gets a list of properties for all entity types

        :param page_size: The amount of entity types in a single response payload.
            The maximal allowed page size is 500.
            If not set, 50 is used.
        :return: A list of properties of all available entity types.
        """
        params = {"pageSize": page_size}
        return PaginatedList(EntityType, self.__http_client, self.ENDPOINT_TYPES, params, list_item="types")

    def get_type(self, entity_type: str) -> "EntityType":
        """Gets the properties of a specified entity type.

        :param entity_type: The entity type required

        :returns EntityType: The properties of the specified entity type.
        """
        response = self.__http_client.make_request(path=f"{self.ENDPOINT_TYPES}/{entity_type}")
        return EntityType(raw_element=response.json())


class Entity(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.last_seen: Optional[datetime] = int64_to_datetime(raw_element.get("lastSeenTms", 0))
        self.first_seen: Optional[datetime] = int64_to_datetime(raw_element.get("firstSeenTms", 0))

        self.from_relationships: Dict[str, List["EntityId"]] = {
            key: [EntityId(raw_element=entity) for entity in entities] for key, entities in raw_element.get("fromRelationships", {}).items()
        }
        self.to_relationships: Dict[str, List["EntityId"]] = {
            key: [EntityId(raw_element=entity) for entity in entities] for key, entities in raw_element.get("toRelationships", {}).items()
        }
        self.management_zones: List[ManagementZone] = [ManagementZone(raw_element=m) for m in raw_element.get("managementZones", [])]
        self.icon: Optional[EntityIcon] = EntityIcon(raw_element=raw_element.get("icon")) if raw_element.get("icon") else None
        self.display_name: str = raw_element["displayName"]
        self.entity_id: str = raw_element["entityId"]
        self.properties: Optional[Dict[str, Any]] = raw_element.get("properties", {})
        self.tags: List[METag] = [METag(raw_element=tag) for tag in raw_element.get("tags", [])]


class EntityShortRepresentation(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.id = raw_element.get("id")
        self.name = raw_element.get("name")
        self.description = raw_element.get("description")

    def to_json(self) -> Dict[str, Any]:
        return {"id": self.id, "name": self.name, "description": self.description}


class EntityStub(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.entity_id: EntityId = EntityId(raw_element=raw_element["entityId"])
        self.name: Optional[str] = raw_element.get("name")

    def to_json(self) -> Dict[str, Any]:
        return {"entityId": self.entity_id.to_json(), "name": self.name}


class EntityId(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.id: str = raw_element["id"]
        self.type: str = raw_element["type"]

    def to_json(self) -> Dict[str, Any]:
        return {"id": self.id, "type": self.type}


class EntityIcon(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.primary_icon_type = raw_element.get("primaryIconType")
        self.secondary_icon_type = raw_element.get("secondaryIconType")
        self.custom_icon_path = raw_element.get("customIconPath")


class CustomDeviceCreation(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.custom_device_id: str = raw_element["customDeviceId"]
        self.display_name: str = raw_element["displayName"]
        self.group: Optional[str] = raw_element.get("group")
        self.ip_addresses: Optional[List[str]] = raw_element.get("ipAddresses")
        self.listen_ports: Optional[List[int]] = raw_element.get("listenPorts")
        self.type: Optional[str] = raw_element.get("type")
        self.favicon_url: Optional[str] = raw_element.get("faviconUrl")
        self.config_url: Optional[str] = raw_element.get("configUrl")
        self.properties: Optional[Dict[str, str]] = raw_element.get("properties")
        self.dns_names: Optional[List[str]] = raw_element.get("dnsNames")
        self.message_type: Optional[MessageType] = MessageType(raw_element.get("messageType"))

    def to_json(self):
        return {
            "customDeviceId": self.custom_device_id,
            "displayName": self.display_name,
            "group": self.group,
            "ipAddresses": self.ip_addresses,
            "listenPorts": self.listen_ports,
            "type": self.type,
            "faviconUrl": self.favicon_url,
            "configUrl": self.config_url,
            "properties": self.properties,
            "dnsNames": self.dns_names,
            "messageType": str(self.message_type),
        }

    def post(self) -> "Response":
        return self._http_client.make_request(path=f"{EntityService.ENDPOINT_ENTITIES}/custom", method="POST", params=self.to_json())


class EntityType(DynatraceObject):
    def _create_from_raw_data(self, raw_element: dict):
        self.type: str = raw_element["type"]
        self.display_name: str = raw_element["displayName"]
        self.entity_limit_exceeded: bool = raw_element.get("entityLimitExceeded", False)
        self.properties: Optional[List[EntityTypePropertyDto]] = [EntityTypePropertyDto(raw_element=p) for p in raw_element.get("properties", [])]
        self.from_relationships: Optional[List[FromPosition]] = [FromPosition(raw_element=fr) for fr in raw_element.get("fromRelationships", [])]
        self.to_relationships: Optional[List[ToPosition]] = [ToPosition(raw_element=tr) for tr in raw_element.get("toRelationships", [])]
        self.dimension_key: Optional[str] = raw_element.get("dimensionKey")
        self.management_zones: Optional[str] = raw_element.get("managementZones")
        self.tags: Optional[str] = raw_element.get("tags")


class EntityTypePropertyDto(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.id: str = raw_element["id"]
        self.type: str = raw_element["type"]
        self.display_name: Optional[str] = raw_element.get("displayName")


class ToPosition(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.from_types: List[str] = raw_element["fromTypes"]
        self.id: str = raw_element["id"]


class FromPosition(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.to_types: List[str] = raw_element["toTypes"]
        self.id: str = raw_element["id"]


class MessageType(Enum):
    CUSTOM_DEVICE = "CUSTOM_DEVICE"
    DELETE_ENTITY_PER_TYPE = "DELETE_ENTITY_PER_TYPE"
    FILTER_VALUE_SUGGESTIONS = "FILTER_VALUE_SUGGESTIONS"
    MULTI = "MULTI"
    MULTI_TYPE = "MULTI_TYPE"
    SINGLE = "SINGLE"
    SINGLE_TYPE = "SINGLE_TYPE"

    def __repr__(self):
        return self.value

    def __str__(self):
        return self.value



"""
Copyright 2021 Dynatrace LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from enum import Enum
from typing import Optional, Union, Any, Dict, List

from datetime import datetime, timedelta

from dynatrace.dynatrace_object import DynatraceObject
from dynatrace.http_client import HttpClient
from dynatrace.pagination import PaginatedList
from dynatrace.utils import timestamp_to_string
from dynatrace.environment_v2.schemas import VersionCompareType


class UpdateType(Enum):
    ACTIVE_GATE = "ACTIVE_GATE"
    REMOTE_PLUGIN_AGENT = "REMOTE_PLUGIN_AGENT"
    SYNTHETIC = "SYNTHETIC"
    Z_REMOTE = "Z_REMOTE"


class ActiveGateAutoUpdateJobsService:
    def __init__(self, http_client: HttpClient):
        self.__http_client = http_client

    def list(
        self,
        time_from: Optional[Union[datetime, str]] = None,
        time_to: Optional[Union[datetime, str]] = None,
        start_version_compare_type: Optional[Union[str, VersionCompareType]] = None,
        start_version: Optional[str] = None,
        update_type: Optional[Union[str, UpdateType]] = None,
        target_version_compare_type: Optional[Union[str, VersionCompareType]] = None,
        target_version: Optional[str] = None,
    ) -> PaginatedList["UpdateJobList"]:
        params = {
            "from": timestamp_to_string(time_from),
            "to": timestamp_to_string(time_to),
            "startVersionCompareType": start_version_compare_type,
            "startVersion": start_version,
            "updateType": update_type,
            "targetVersionCompareType": target_version_compare_type,
            "targetVersion": target_version,
        }
        return PaginatedList(UpdateJobList, self.__http_client, f"/api/v2/activeGates/updateJobs", list_item="allUpdateJobs", target_params=params)

    def get(
        self,
        activegate_id: str,
        time_from: Optional[Union[datetime, str]] = None,
        time_to: Optional[Union[datetime, str]] = None,
        start_version_compare_type: Optional[Union[str, VersionCompareType]] = None,
        start_version: Optional[str] = None,
        update_type: Optional[Union[str, UpdateType]] = None,
        target_version_compare_type: Optional[Union[str, VersionCompareType]] = None,
        target_version: Optional[str] = None,
    ) -> "UpdateJobList":
        params = {
            "from": timestamp_to_string(time_from),
            "to": timestamp_to_string(time_to),
            "startVersionCompareType": start_version_compare_type,
            "startVersion": start_version,
            "updateType": update_type,
            "targetVersionCompareType": target_version_compare_type,
            "targetVersion": target_version,
        }
        return UpdateJobList(raw_element=self.__http_client.make_request(f"/api/v2/activeGates/{activegate_id}/updateJobs", params=params).json())

    def post(self, activegate_id: str, target_version: str):
        params = {"targetVersion": target_version}
        return UpdateJob(raw_element=self.__http_client.make_request(f"/api/v2/activeGates/{activegate_id}/updateJobs", params=params, method="POST").json())

    def validate(self, activegate_id: str, target_version: str):
        params = {"targetVersion": target_version}
        return self.__http_client.make_request(f"/api/v2/activeGates/{activegate_id}/updateJobs/validator", params=params, method="POST")

    def get_job(self, activegate_id: str, job_id: str):
        return UpdateJob(raw_element=self.__http_client.make_request(f"/api/v2/activeGates/{activegate_id}/updateJobs/{job_id}").json())

    def delete_job(self, activegate_id: str, job_id: str):
        return self.__http_client.make_request(f"/api/v2/activeGates/{activegate_id}/updateJobs/{job_id}", method="DELETE")


class UpdateJobList(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.activegate_id: str = raw_element.get("agId")
        self.update_jobs: List["UpdateJob"] = [UpdateJob(raw_element=update_job) for update_job in raw_element.get("updateJobs", [])]


class UpdateJob(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.job_id: str = raw_element.get("jobId")
        self.job_state: str = raw_element.get("jobState")
        self.update_method: str = raw_element.get("updateMethod")
        self.update_type: str = raw_element.get("updateType")
        self.cancelable: bool = raw_element.get("cancelable")
        self.start_version: str = raw_element.get("startVersion")
        self.target_version: str = raw_element.get("targetVersion")
        self.timestamp: datetime = datetime.utcfromtimestamp(raw_element.get("timestamp") / 1000)
        self.ag_type: str = raw_element.get("agType")
        self.environments: List[str] = raw_element.get("environments")
        self.error: str = raw_element.get("error")
        self.duration: Optional[timedelta] = timedelta(milliseconds=raw_element.get("duration")) if raw_element.get("duration") else None



from dynatrace import Dynatrace
from dynatrace.environment_v2.service_level_objectives import Slo, SloStatus, SloError, SloEvaluationType
from dynatrace.pagination import PaginatedList

SLO_ID = "d4adc421-245e-3bc5-a683-df2ed030997c"


def test_list(dt: Dynatrace):
    slos = dt.slos.list(page_size=20, evaluate=True)

    assert isinstance(slos, PaginatedList)
    assert len(list(slos)) == 2
    assert all(isinstance(s, Slo) for s in slos)


def test_get(dt: Dynatrace):
    slo = dt.slos.get(slo_id=SLO_ID)

    # type checks
    assert isinstance(slo, Slo)
    assert isinstance(slo.enabled, bool)
    assert isinstance(slo.name, str)
    assert isinstance(slo.custom_description, str)
    assert isinstance(slo.evaluated_percentage, float)
    assert isinstance(slo.error_budget, float)
    assert isinstance(slo.status, SloStatus)
    assert isinstance(slo.error, SloError)
    assert isinstance(slo.use_rate_metric, bool)
    assert isinstance(slo.metric_rate, str)
    assert isinstance(slo.metric_numerator, str)
    assert isinstance(slo.metric_denominator, str)
    assert isinstance(slo.numerator_value, (float, int))
    assert isinstance(slo.denominator_value, (float, int))
    assert isinstance(slo.target, float)
    assert isinstance(slo.warning, float)
    assert isinstance(slo.evaluation_type, SloEvaluationType)
    assert isinstance(slo.timeframe, str)
    assert isinstance(slo.filter, str)
    assert isinstance(slo.related_open_problems, int)

    # value checks
    assert slo.id == SLO_ID
    assert slo.enabled == True
    assert slo.name == "MySLOService"
    assert slo.custom_description == "Service Errors Fivexx SuccessCount / Service RequestCount Total"
    assert slo.evaluated_percentage == 99.92798959015639
    assert slo.error_budget == -0.022010409843616685
    assert slo.status == SloStatus.FAILURE
    assert slo.error == SloError.NONE
    assert slo.use_rate_metric == False
    assert slo.metric_rate == ""
    assert slo.metric_numerator == "builtin:service.errors.fivexx.successCount:splitBy()"
    assert slo.metric_denominator == "builtin:service.requestCount.total:splitBy()"
    assert slo.numerator_value == 1704081
    assert slo.denominator_value == 1705309
    assert slo.target == 99.95
    assert slo.warning == 99.97
    assert slo.evaluation_type == SloEvaluationType.AGGREGATE
    assert slo.timeframe == "-2h"
    assert slo.filter == "type(SERVICE),entityId(SERVICE-D89AF859A68D9072)"
    assert slo.related_open_problems == 0



"""
Copyright 2021 Dynatrace LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from requests import Response
from typing import Dict, Any, Optional, Union

from dynatrace.http_client import HttpClient
from dynatrace.dynatrace_object import DynatraceObject
from dynatrace.environment_v2.schemas import ConfigurationMetadata
from dynatrace.configuration_v1.schemas import AutoUpdateSetting, UpdateWindowsConfig, EffectiveSetting


class OneAgentInAHostGroupService:
    ENDPOINT = "/api/config/v1/hostgroups"

    def __init__(self, http_client: HttpClient):
        self.__http_client = http_client

    def get(self, hostgroup_id: str) -> "OneAgentHostGroupConfig":
        """Gets the OneAgent configuration in the specified host group

        :param hostgroup_id: The Dynatrace entity ID of the required host group.

        :returns OneAgentHostGroupConfig: the full OneAgent configuration in this host group
        """
        response = self.__http_client.make_request(path=f"{self.ENDPOINT}/{hostgroup_id}").json()
        return OneAgentHostGroupConfig(raw_element=response, http_client=self.__http_client)

    def get_autoupdate(self, hostgroup_id: str) -> "HostGroupAutoUpdateConfig":
        """Gets the configuration of OneAgent auto-update in the specified host group

        :param hostgroup_id: The Dynatrace entity ID of the required host group.

        :returns HostGroupAutoUpdateConfig: The The auto-update configuration details in this host group
        """
        response = self.__http_client.make_request(path=f"{self.ENDPOINT}/{hostgroup_id}/autoupdate").json()
        return HostGroupAutoUpdateConfig(raw_element=response, http_client=self.__http_client)

    def put_autoupdate(self, hostgroup_id: str, config: "HostGroupAutoUpdateConfig") -> "Response":
        """Updates the configuration of OneAgent auto-update in the specified host group

        OneAgents are updated several minutes after the change of configuration.
        The process can take more time depending on number of OneAgents in the update queue.

        :param hostgroup_id: The Dynatrace entity ID of the required host group.
        :param config: he updated auto-update configuration object

        :returns Response: HTTP Response to the request
        """
        return self.__http_client.make_request(path=f"{self.ENDPOINT}/{hostgroup_id}/autoupdate", method="PUT", params=config.to_json())

    def is_valid_autoupdate(self, hostgroup_id: str, config: "HostGroupAutoUpdateConfig") -> bool:
        """Validates the payload for the put_autoupdate function

        :param hostgroup_id: The Dynatrace entity ID of the required host group.
        :param config: the auto-update configuration object to validate

        :returns bool: True if valid, False otherwise
        """
        try:
            self.__http_client.make_request(path=f"{self.ENDPOINT}/{hostgroup_id}/autoupdate/validator", method="POST", params=config.to_json())
        except Exception as e:
            print(e.args)
            return False
        else:
            return True


class OneAgentHostGroupConfig(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.id: Optional[str] = raw_element.get("id")
        self.auto_update_config: HostGroupAutoUpdateConfig = HostGroupAutoUpdateConfig(
            raw_element=raw_element.get("autoUpdateConfig"), http_client=self._http_client
        )


class HostGroupAutoUpdateConfig(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.metadata: ConfigurationMetadata = ConfigurationMetadata(raw_element=raw_element.get("metadata"))
        self.id: str = raw_element.get("id", "")
        self.setting: AutoUpdateSetting = AutoUpdateSetting(raw_element.get("setting"))
        self.update_windows: UpdateWindowsConfig = UpdateWindowsConfig(raw_element.get("updateWindows"))
        self.effective_setting: Optional[EffectiveSetting] = None
        self.version: Optional[str] = raw_element.get("version")
        self.effective_version: Optional[str] = raw_element.get("effectiveVersion")

        if raw_element.get("effectiveSetting"):
            self.effective_setting = EffectiveSetting(raw_element.get("effectiveSetting"))

    def to_json(self) -> Dict[str, Any]:
        return {
            "setting": str(self.setting),
            "version": self.version,
            "updateWindows": self.update_windows.to_json(),
            "effectiveVersion": self.effective_version,
        }

    def put(self) -> "Response":
        return self._http_client.make_request(path=f"{OneAgentInAHostGroupService.ENDPOINT}/{self.id}/autoupdate", method="PUT", params=self.to_json())



from dynatrace import Dynatrace

from dynatrace.environment_v1.synthetic_monitors import CreatedFrom, ManagementZone, MonitorCollectionElement, MonitorType
from dynatrace.pagination import PaginatedList


def test_list_string(dt: Dynatrace):
    monitors = dt.synthetic_monitors.list(monitor_type="BROWSER")
    assert isinstance(monitors, PaginatedList)
    for monitor in monitors:
        assert isinstance(monitor, MonitorCollectionElement)
        assert monitor.name == "angular easytravel bounce"
        assert monitor.entity_id == "SYNTHETIC_TEST-7639A3AED66940FA"
        assert monitor.monitor_type == "BROWSER"
        assert monitor.enabled
        break


def test_list_enum(dt: Dynatrace):
    monitors = dt.synthetic_monitors.list(monitor_type=MonitorType.BROWSER)
    assert isinstance(monitors, PaginatedList)
    for monitor in monitors:
        assert isinstance(monitor, MonitorCollectionElement)
        assert monitor.name == "angular easytravel bounce"
        assert monitor.entity_id == "SYNTHETIC_TEST-7639A3AED66940FA"
        assert monitor.monitor_type == "BROWSER"
        assert monitor.enabled
        break


def test_get_full_synthetic_config(dt: Dynatrace):
    config = dt.synthetic_monitors.get_full_monitor_configuration(monitor_id="SYNTHETIC_TEST-7639A3AED66940FA")
    assert config.name == "angular easytravel bounce"
    assert config.frequency_min == 5
    assert config.enabled
    assert config.type == MonitorType.BROWSER
    assert config.created_from == CreatedFrom.API
    assert isinstance(config.script, dict)
    assert config.anomaly_detection.outage_handling.local_outage_policy.consecutive_runs == 1
    assert not config.anomaly_detection.loading_time_thresholds.enabled
    for mz in config.management_zones:
        assert isinstance(mz, ManagementZone)
        break



from datetime import datetime

from dynatrace import Dynatrace
from dynatrace.configuration_v1.extensions import (
    ExtensionDto,
    ExtensionType,
    Extension,
    ExtensionProperty,
    GlobalExtensionConfiguration,
    ExtensionState,
    ExtensionStateEnum,
    ExtensionConfigurationDto,
)
from dynatrace.environment_v2.monitored_entities import EntityShortRepresentation
from dynatrace.pagination import PaginatedList


def test_list(dt: Dynatrace):
    extensions = dt.extensions.list()
    assert isinstance(extensions, PaginatedList)

    extensions_list = list(extensions)
    assert len(extensions_list) == 35
    first = extensions_list[0]

    assert isinstance(first, ExtensionDto)
    assert first.id == "custom.remote.python.certificates"
    assert first.name == "Certificates Plugin"
    assert first.type == ExtensionType.ACTIVEGATE


def test_get(dt: Dynatrace):
    extension = dt.extensions.get("custom.python.citrixAgent")
    assert isinstance(extension, Extension)
    assert extension.id == "custom.python.citrixAgent"
    assert extension.name == "Citrix Virtual Apps & Virtual Desktops"
    assert extension.version == "2.034"
    assert extension.type == ExtensionType.ONEAGENT
    assert extension.metric_group == "tech.Citrix"
    assert isinstance(extension.properties, list)

    first_property = extension.properties[0]
    assert isinstance(first_property, ExtensionProperty)
    assert first_property.key == "openkit_verify_certificates"
    assert first_property.type == "BOOLEAN"


def test_get_global_configuration(dt: Dynatrace):
    global_config = dt.extensions.get_global_configuration("custom.python.citrixAgent")
    assert isinstance(global_config, GlobalExtensionConfiguration)
    assert global_config.extension_id == "custom.python.citrixAgent"
    assert global_config.enabled
    assert not global_config.infraOnlyEnabled
    assert global_config.properties["log_level"] == "INFO"


def test_get_state(dt: Dynatrace):
    states = dt.extensions.list_states("custom.remote.python.salesforce_eventstream")
    assert isinstance(states, PaginatedList)

    list_states = list(states)
    assert isinstance(list_states, list)

    first = list_states[0]
    assert isinstance(first, ExtensionState)
    assert first.extension_id == "custom.remote.python.salesforce_eventstream"
    assert first.version == ""
    assert first.endpoint_id == "5649014104314746667"
    assert first.state == ExtensionStateEnum.ERROR_CONFIG
    assert first.state_description == "Extension doesn't exist on given host"
    assert first.timestamp == datetime.utcfromtimestamp(1620943873929 / 1000)
    assert first.host_id is None
    assert first.process_id is None


def test_get_instance_configuration(dt: Dynatrace):
    config = dt.extensions.get_instance_configuration("custom.remote.python.salesforce_eventstream", "5649014104314746667")
    assert isinstance(config, ExtensionConfigurationDto)

    # TODO - This is a bug on Dynatrace, watch for the fix, this is the configuration ID
    assert config.extension_id == "5649014104314746667"

    assert config.enabled
    assert config.active_gate.id == "-7885258652650793909"
    assert config.active_gate.name == "arch-david"
    assert config.endpoint_id == "5649014104314746667"
    assert config.endpoint_name == "curious-hawk"
    assert config.properties["openkit_application_id"] == "87eee414-9338-446b-988b-bbdbf495c4f4"


def test_list_activegate_extension_modules(dt: Dynatrace):
    modules = dt.extensions.list_activegate_extension_modules()
    assert isinstance(modules, PaginatedList)

    list_modules = list(modules)
    assert isinstance(list_modules, list)

    first = list_modules[0]
    assert isinstance(first, EntityShortRepresentation)
    assert first.id == "-7885258652650793909"
    assert first.name == "arch-david"



from dynatrace import Dynatrace
from dynatrace.configuration_v1.oneagent_environment_wide_configuration import EnvironmentAutoUpdateConfig
from dynatrace.configuration_v1.schemas import (
    ConfigurationMetadata,
    AutoUpdateSetting,
    UpdateWindowsConfig,
    TechMonitoringList,
    Technology,
    TechnologyType,
    SettingScope,
)

CLUSTER_VERSION = "1.222.47.20210712-162143"


def test_get(dt: Dynatrace):
    oa_env_autoupdate_config = dt.oneagents_config_environment.get()

    # type checks
    assert isinstance(oa_env_autoupdate_config, EnvironmentAutoUpdateConfig)
    assert isinstance(oa_env_autoupdate_config.metadata, ConfigurationMetadata)
    assert isinstance(oa_env_autoupdate_config.setting, AutoUpdateSetting)
    assert isinstance(oa_env_autoupdate_config.version, (str, type(None)))
    assert isinstance(oa_env_autoupdate_config.update_windows, UpdateWindowsConfig)

    # value checks
    assert oa_env_autoupdate_config.metadata.cluster_version == CLUSTER_VERSION
    assert oa_env_autoupdate_config.setting == AutoUpdateSetting.DISABLED
    assert oa_env_autoupdate_config.version is None
    assert oa_env_autoupdate_config.update_windows.windows == []


def test_get_technologies(dt: Dynatrace):
    oa_env_technologies = dt.oneagents_config_environment.get_technologies()

    # type checks
    assert isinstance(oa_env_technologies, TechMonitoringList)
    assert isinstance(oa_env_technologies.metadata, ConfigurationMetadata)
    assert all(isinstance(t, Technology) for t in oa_env_technologies.technologies)
    for tech in oa_env_technologies.technologies:
        assert isinstance(tech.type, TechnologyType)
        assert isinstance(tech.monitoring_enabled, bool)
        assert isinstance(tech.scope, (SettingScope, type(None)))

    # value checks
    assert oa_env_technologies.metadata.cluster_version == CLUSTER_VERSION
    assert len(oa_env_technologies.technologies) == 4
    assert oa_env_technologies.technologies[0].type == TechnologyType.DOT_NET
    assert oa_env_technologies.technologies[0].monitoring_enabled == True
    assert oa_env_technologies.technologies[0].scope == SettingScope.ENVIRONMENT



from dynatrace import Dynatrace
from dynatrace.configuration_v1.oneagent_in_a_hostgroup import OneAgentHostGroupConfig, HostGroupAutoUpdateConfig
from dynatrace.configuration_v1.schemas import UpdateWindowsConfig, UpdateWindow, ConfigurationMetadata, EffectiveSetting, AutoUpdateSetting

HOST_GROUP_ID = "HOST_GROUP-ABC123DEF456GHI7"
CLUSTER_VERSION = "1.222.47.20210712-162143"


def test_get(dt: Dynatrace):
    oa_hostgroup_config = dt.oneagents_config_hostgroup.get(HOST_GROUP_ID)

    # type checks
    assert isinstance(oa_hostgroup_config, OneAgentHostGroupConfig)
    assert isinstance(oa_hostgroup_config.id, (str, type(None)))
    assert isinstance(oa_hostgroup_config.auto_update_config, HostGroupAutoUpdateConfig)

    # value checks
    assert oa_hostgroup_config.id == None


def test_get_audoupdate(dt: Dynatrace):
    oa_hostgroup_autoupdate = dt.oneagents_config_hostgroup.get_autoupdate(HOST_GROUP_ID)

    # type checks
    assert isinstance(oa_hostgroup_autoupdate, HostGroupAutoUpdateConfig)
    assert isinstance(oa_hostgroup_autoupdate.id, str)
    assert isinstance(oa_hostgroup_autoupdate.metadata, ConfigurationMetadata)
    assert isinstance(oa_hostgroup_autoupdate.setting, AutoUpdateSetting)
    assert isinstance(oa_hostgroup_autoupdate.update_windows, UpdateWindowsConfig)
    assert isinstance(oa_hostgroup_autoupdate.effective_setting, (EffectiveSetting, type(None)))
    assert isinstance(oa_hostgroup_autoupdate.version, (str, type(None)))
    assert isinstance(oa_hostgroup_autoupdate.effective_version, (str, type(None)))

    # value checks
    assert oa_hostgroup_autoupdate.metadata.cluster_version == CLUSTER_VERSION
    assert oa_hostgroup_autoupdate.id == HOST_GROUP_ID
    assert oa_hostgroup_autoupdate.setting == AutoUpdateSetting.DISABLED
    assert oa_hostgroup_autoupdate.version is None
    assert oa_hostgroup_autoupdate.update_windows.windows == []
    assert oa_hostgroup_autoupdate.effective_setting == EffectiveSetting.DISABLED
    assert oa_hostgroup_autoupdate.effective_version is None



"""
Copyright 2021 Dynatrace LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from typing import List, Optional
from requests import Response

from dynatrace.dynatrace_object import DynatraceObject
from dynatrace.environment_v2.monitored_entities import EntityShortRepresentation
from dynatrace.http_client import HttpClient
from dynatrace.pagination import PaginatedList
from dynatrace.configuration_v1.tile import Tile


class DashboardService:
    def __init__(self, http_client: HttpClient):
        self.__http_client = http_client

    def list(self, owner: str = None, tags: List[str] = None) -> PaginatedList["DashboardStub"]:
        """
        Lists all dashboards of the environment
        :param owner: The owner of the dashboard.
        :param tags: A list of tags applied to the dashboard.
            The dashboard must match all the specified tags.
        """
        params = {"owner": owner, "tags": tags}
        return PaginatedList(DashboardStub, self.__http_client, f"/api/config/v1/dashboards", params, list_item="dashboards")

    def get(self, dashboard_id: str) -> "Dashboard":
        """
        Gets the properties of the specified dashboard
        """
        response = self.__http_client.make_request(f"/api/config/v1/dashboards/{dashboard_id}").json()
        return Dashboard(self.__http_client, None, response)

    def delete(self, dashboard_id: str) -> Response:
        """
        Deletes the specified dashboard
        """
        return self.__http_client.make_request(f"/api/config/v1/dashboards/{dashboard_id}", method="DELETE")


class DashboardFilter(DynatraceObject):
    def _create_from_raw_data(self, raw_element):
        if raw_element is None:
            raw_element = {}
        self.timeframe: str = raw_element.get("timeframe")
        self.management_zone: Optional[EntityShortRepresentation] = (
            EntityShortRepresentation(self._http_client, None, raw_element.get("managementZone")) if raw_element.get("managementZone") else None
        )


class DashboardMetadata(DynatraceObject):
    def _create_from_raw_data(self, raw_element):
        # TODO - Schema changed, add extra fields dynamicFilters
        self.name: str = raw_element.get("name")
        self.shared: bool = raw_element.get("shared")
        self.owner: str = raw_element.get("owner")
        self.dashboard_filter: DashboardFilter = DashboardFilter(self._http_client, None, raw_element.get("dashboardFilter"))
        self.tags: List[str] = raw_element.get("tags")
        self.preset: bool = raw_element.get("preset")


class Dashboard(DynatraceObject):
    def _create_from_raw_data(self, raw_element):
        if raw_element is None:
            raw_element = {}
        self.id: str = raw_element.get("id")
        self.dashboard_metadata: DashboardMetadata = DashboardMetadata(self._http_client, None, raw_element.get("dashboardMetadata"))
        self.tiles: List[Tile] = [Tile(self._http_client, None, raw_tile) for raw_tile in raw_element.get("tiles", [])]


class DashboardStub(DynatraceObject):
    def delete(self) -> Response:
        """
        Deletes this dashboard
        """
        return self._http_client.make_request(f"/api/config/v1/dashboards/{self.id}", method="DELETE")

    def _create_from_raw_data(self, raw_element):
        self.id: str = raw_element.get("id")
        self.name: str = raw_element.get("name")
        self.owner: str = raw_element.get("owner")

    def get_full_dashboard(self) -> Dashboard:
        """
        Gets the full dashboard for this stub
        """
        response = self._http_client.make_request(f"/api/config/v1/dashboards/{self.id}").json()
        return Dashboard(self._http_client, None, response)



"""
Copyright 2021 Dynatrace LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from enum import Enum
from typing import List, Optional, Union
from datetime import datetime

from dynatrace.pagination import PaginatedList
from dynatrace.dynatrace_object import DynatraceObject
from dynatrace.http_client import HttpClient
from dynatrace.utils import datetime_to_int64


class ConfiguredMonitoringMode(Enum):
    CLOUD_INFRASTRUCTURE = "CLOUD_INFRASTRUCTURE"
    FULL_STACK = "FULL_STACK"


class UpdateStatus(Enum):
    INCOMPATIBLE = "INCOMPATIBLE"
    OUTDATED = "OUTDATED"
    SCHEDULED = "SCHEDULED"
    SUPPRESSED = "SUPPRESSED"
    UNKNOWN = "UNKNOWN"
    UP2DATE = "UP2DATE"
    UPDATE_IN_PROGRESS = "UPDATE_IN_PROGRESS"
    UPDATE_PENDING = "UPDATE_PENDING"
    UPDATE_PROBLEM = "UPDATE_PROBLEM"


class MonitoringType(Enum):
    CLOUD_INFRASTRUCTURE = "CLOUD_INFRASTRUCTURE"
    FULL_STACK = "FULL_STACK"
    STANDALONE = "STANDALONE"


class AvailabilityState(Enum):
    MONITORED = "MONITORED"
    UNMONITORED = "UNMONITORED"
    CRASHED = "CRASHED"
    LOST = "LOST"
    PRE_MONITORED = "PRE_MONITORED"
    SHUTDOWN = "SHUTDOWN"
    UNEXPECTED_SHUTDOWN = "UNEXPECTED_SHUTDOWN"
    UNKNOWN = "UNKNOWN"


class AutoUpdate(Enum):
    ENABLED = "ENABLED"
    DISABLED = "DISABLED"


class OsType(Enum):
    AIX = "AIX"
    LINUX = "LINUX"
    WINDOWS = "WINDOWS"
    SOLARIS = "SOLARIS"
    ZOS = "ZOS"


class OneAgentOnAHostService:
    def __init__(self, http_client: HttpClient):
        self.__http_client = http_client

    def list(
        self,
        include_details: Optional[bool] = None,
        start_timestamp: Optional[Union[datetime, int]] = None,
        end_timestamp: Optional[Union[datetime, int]] = None,
        relative_time: Optional[str] = None,
        tag: Optional[List[str]] = None,
        entity: Optional[List[str]] = None,
        mz_id: Optional[str] = None,
        management_zone: Optional[str] = None,
        network_zone_id: Optional[str] = None,
        host_group_id: Optional[str] = None,
        host_group_name: Optional[str] = None,
        os_type: Optional[Union[OsType, str]] = None,
        availability_state: Optional[Union[AvailabilityState, str]] = None,
        monitoring_type: Optional[Union[MonitoringType, str]] = None,
        auto_update: Optional[Union[AutoUpdate, str]] = None,
    ) -> PaginatedList["HostAgentInfo"]:
        """
        Lists OneAgents on a Host. Older API - timestamps are of type integer and therefore require conversion
        """
        params = {
            "includeDetails": include_details,
            "startTimestamp": datetime_to_int64(start_timestamp),
            "endTimestamp": datetime_to_int64(end_timestamp),
            "relativeTime": relative_time,
            "tag": tag,
            "entity": entity,
            "managementZoneId": mz_id,
            "managementZone": management_zone,
            "networkZoneId": network_zone_id,
            "hostGroupId": host_group_id,
            "hostGroupName": host_group_name,
            "osType": OsType(os_type).value if os_type else None,
            "availabilityState": AvailabilityState(availability_state).value if availability_state else None,
            "monitoringType": MonitoringType(monitoring_type).value if monitoring_type else None,
            "autoUpdateSetting": AutoUpdate(auto_update).value if auto_update else None,
        }
        return PaginatedList(HostAgentInfo, self.__http_client, "/api/v1/oneagents", params, list_item="hosts")


# todo - create class objects for ModuleInfo[] and PluginInfo[]
class HostAgentInfo(DynatraceObject):
    def _create_from_raw_data(self, raw_element):
        self.host_info: HostInfo = HostInfo(raw_element=raw_element.get("hostInfo"))
        self.faulty_version: bool = raw_element.get("faultyVersion")
        self.active: bool = raw_element.get("active")
        self.configured_monitoring_mode: ConfiguredMonitoringMode = ConfiguredMonitoringMode(raw_element.get("configuredMonitoringMode"))
        self.monitoring_type: MonitoringType = MonitoringType(raw_element.get("monitoringType"))
        self.auto_update: AutoUpdate = AutoUpdate(raw_element.get("autoUpdateSetting"))
        self.update_status: UpdateStatus = UpdateStatus(raw_element.get("updateStatus"))
        self.available_versions: str = raw_element.get("availableVersions", [])
        self.config_monitoring_enabled: bool = raw_element.get("configuredMonitoringEnabled")
        self.availability_state: AvailabilityState = AvailabilityState(raw_element.get("availabilityState"))
        self.activegate_id: int = raw_element.get("currentActiveGateId")
        self.networkzone_id: str = raw_element.get("currentNetworkZoneId")


# todo - incomplete + firstSeenTimestamp is of type integer, how do we work with that here?
class HostInfo(DynatraceObject):
    def _create_from_raw_data(self, raw_element):
        self.entity_id: str = raw_element.get("entityId")
        self.display_name: str = raw_element.get("displayName")
        self.discovered_name: str = raw_element.get("discoveredName")
        self.consumed_host_units: str = raw_element.get("consumedHostUnits")
        self.os_version: str = raw_element.get("osVersion")
        self.host_group: HostGroup = HostGroup(raw_element=raw_element.get("hostGroup"))
        self.tags: List[TagInfo] = [TagInfo(raw_element=t) for t in raw_element.get("tags", [])]
        self.os_type: Optional[OsType] = OsType(raw_element.get("osType")) if raw_element.get("osType") else None


class HostGroup(DynatraceObject):
    def _create_from_raw_data(self, raw_element):
        self.me_id: str = raw_element.get("meId")
        self.name: str = raw_element.get("name")


class TagInfo(DynatraceObject):
    # todo - convert context to Enum
    def _create_from_raw_data(self, raw_element):
        self.context: str = raw_element.get("context")
        self.key: str = raw_element.get("key")
        self.value: str = raw_element.get("value")



from dynatrace import Dynatrace


def test_get_global(dt: Dynatrace):
    alert_profiles = dt.activegates_autoupdate_configuration.get_global()
    assert alert_profiles.global_setting == "DISABLED"
    assert alert_profiles.metadata.cluster_version == "1.217.96.20210507-181038"
    assert alert_profiles.metadata.configuration_versions[0] == 1
    assert isinstance(alert_profiles.metadata.configuration_versions, list)


def test_get(dt: Dynatrace):
    alert_profiles = dt.activegates_autoupdate_configuration.get("1513404008")
    assert alert_profiles.setting == "ENABLED"
    assert alert_profiles.effective_setting == "ENABLED"



"""
Copyright 2021 Dynatrace LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from typing import Optional, Dict, List, Any
from requests import Response

from dynatrace.dynatrace_object import DynatraceObject
from dynatrace.http_client import HttpClient


class DeploymentService:
    ENDPOINT_INSTALLER_AGENT = "/api/v1/deployment/installer/agent"
    ENDPOINT_INSTALLER_GATEWAY = "/api/v1/deployment/installer/gateway"
    ENDPOINT_BOSHRELEASE = "/api/v1/deployment/boshrelease"
    ENDPOINT_LAMBDA = "/api/v1/deployment/lambda/agent/latest"
    ENDPOINT_ORCHESTRATION = "/api/v1/deployment/orchestration/agent"

    def __init__(self, http_client: HttpClient):
        self.__http_client = http_client

    def get_agent_installer_latest_metainfo(
        self, os_type: str, installer_type: str, flavor: Optional[str] = None, arch: Optional[str] = None, bitness: Optional[str] = None
    ) -> "InstallerMetaInfoDto":
        """Returns the OneAgent version of the installer of the specified type.
        Non-required parameters are only applicable to the paas and paas-sh installer types.

        :param os_type: The operating system of the installer. Use one of: windows, unix, aix, solaris
        :param installer_type: The type of installer. Use one of:
            - default: Self-extracting installer for manual installation. Downloads an .exe file for Windows or an .sh file for Unix.
            - paas: Code modules installer. Downloads a *.zip archive, containing the manifest.json file with meta information or a .jar file for z/OS.
            - paas-sh: Code modules installer. Downloads a self-extracting shell script with the embedded tar.gz archive. \n
        :param flavor: (only for paas and paas-sh) the flavor of your Linux distribution. Use one of:
            - musl: for Linux distributions, which are using the musl C standard library, for example Alpine Linux.
            - multidistro: for all Linux distributions which are using musl C and glibc standard library. \n
        :param arch: (only for paas and paas-sh) the architecture of your OS. Use one of:
            - all: Use this value for AIX and z/OS. Defaults to x86 for other OS types.
            - x86: x86 architecture.
            - ppc: PowerPC architecture, only supported for AIX and Linux.
            - ppcle: PowerPC Little Endian architecture, only supported for Linux.
            - sparc: Sparc architecture, only supported for Solaris.
            - arm: ARM architecture, only supported for Linux.
            - s390: S/390 architecture, only supported for Linux. \n
        :param bitness: (only for paas and paas-sh) the bitness of your OS. Must be supported by the OS. Use one of:
            - 32
            - 64
            - all \n

        :returns InstallerMetaInfo: the latest version of the installer of that type
        """
        params = {"flavor": flavor, "arch": arch, "bitness": bitness}
        response = self.__http_client.make_request(path=f"{self.ENDPOINT_INSTALLER_AGENT}/{os_type}/{installer_type}/latest/metainfo", params=params)
        return InstallerMetaInfoDto(raw_element=response.json())

    def get_agent_installer(
        self,
        os_type: str,
        installer_type: str,
        version: str = "latest",
        flavor: Optional[str] = None,
        arch: Optional[str] = None,
        bitness: Optional[str] = None,
        include: Optional[List[str]] = None,
        skip_metadata: Optional[bool] = None,
        network_zone: Optional[str] = None,
        if_none_match: Optional[str] = None,
    ) -> "Response":
        """Downloads OneAgent installer of the specified version.
        The installer is avaialable in the "content" attribute of the response.

        :param os_type: The operating system of the installer. Use one of: windows, unix, aix, solaris
        :param installer_type: The type of installer. Use one of:
            - default: Self-extracting installer for manual installation. Downloads an .exe file for Windows or an .sh file for Unix.
            - paas: Code modules installer. Downloads a *.zip archive, containing the manifest.json file with meta information or a .jar file for z/OS.
            - paas-sh: Code modules installer. Downloads a self-extracting shell script with the embedded tar.gz archive. \n
        :param version: The exact version of the OneAgent installer. If none is provided, latest available is used.
        :param flavor: (only for paas and paas-sh) the flavor of your Linux distribution. Use one of:
            - musl: for Linux distributions, which are using the musl C standard library, for example Alpine Linux.
            - multidistro: for all Linux distributions which are using musl C and glibc standard library. \n
        :param arch: (only for paas and paas-sh) the architecture of your OS. Use one of:
            - all: Use this value for AIX and z/OS. Defaults to x86 for other OS types.
            - x86: x86 architecture.
            - ppc: PowerPC architecture, only supported for AIX and Linux.
            - ppcle: PowerPC Little Endian architecture, only supported for Linux.
            - sparc: Sparc architecture, only supported for Solaris.
            - arm: ARM architecture, only supported for Linux.
            - s390: S/390 architecture, only supported for Linux. \n
        :param bitness: (only for paas and paas-sh) the bitness of your OS. Must be supported by the OS. Use one of:
            - 32
            - 64
            - all \n
        :param include: (only for paas and paas-sh) the code modules to be included to the installer (e.g. ['java', 'apache'])
        :param skip_metadata: (only for paas and paas-sh) set true to omit the OneAgent connectivity information from the installer.
        :param network_zone: the network zone you want the result to be configured with.
        :param if_none_match: The ETag of the previous request. Do not download if it matches the ETag of the installer.
            The ETag is available in the headers of the response.

        :returns Response: HTTP Response to the request. Can be written to file from the "content" attribute.
        """
        if version != "latest":
            version = "version/" + version
        params = {
            "flavor": flavor,
            "arch": arch,
            "bitness": bitness,
            "include": "&include=".join(include) if include else None,
            "skipMetadata": skip_metadata,
            "networkZone": network_zone,
        }
        headers = {"If-None-Match": if_none_match} if if_none_match else None
        return self.__http_client.make_request(path=f"{self.ENDPOINT_INSTALLER_AGENT}/{os_type}/{installer_type}/{version}", params=params, headers=headers)

    def get_agent_installer_connection_info(self, network_zone: Optional[str] = "default", version: Optional[str] = None) -> "ConnectionInfo":
        """Gets the connectivity information for OneAgent.

        :param network_zone: The network zone you want the result to be configured with.
        :param version: The version of the OneAgent to which the result will be applied.

        :returns ConnectionInfo: connectivity information
        """
        params = {"networkZone": network_zone, "version": version}
        response = self.__http_client.make_request(path=f"{self.ENDPOINT_INSTALLER_AGENT}/connectioninfo", params=params)

        return ConnectionInfo(raw_element=response.json())

    def get_agent_installer_connection_endpoints(self, network_zone: Optional[str] = "default") -> str:
        """Gets the list of the ActiveGate-Endpoints to be used for Agents.
        Ordered by networkzone-priorities. Highest priority first, separated by a semicolon.
        Responds with 404 if network zone is not known.

        :param network_zone: The network zone you want the result to be configured with.

        :returns str: ActiveGate Endpoints separated by semicolons
        """
        params = {"networkZone": network_zone}
        return self.__http_client.make_request(path=f"{self.ENDPOINT_INSTALLER_AGENT}/connectioninfo/endpoints", params=params).text

    def list_agent_installer_versions(
        self, os_type: str, installer_type: str, flavor: Optional[str] = None, arch: Optional[str] = None
    ) -> "AgentInstallerVersions":
        """Lists all available versions of OneAgent installer

        :param os_type: The operating system of the installer. Use one of: windows, unix, aix, solaris
        :param installer_type: The type of installer. Use one of:
            - default: Self-extracting installer for manual installation. Downloads an .exe file for Windows or an .sh file for Unix.
            - paas: Code modules installer. Downloads a *.zip archive, containing the manifest.json file with meta information or a .jar file for z/OS.
            - paas-sh: Code modules installer. Downloads a self-extracting shell script with the embedded tar.gz archive. \n
        :param flavor: (only for paas and paas-sh) the flavor of your Linux distribution. Use one of:
            - musl: for Linux distributions, which are using the musl C standard library, for example Alpine Linux.
            - multidistro: for all Linux distributions which are using musl C and glibc standard library. \n
        :param arch: (only for paas and paas-sh) the architecture of your OS. Use one of:
            - all: Use this value for AIX and z/OS. Defaults to x86 for other OS types.
            - x86: x86 architecture.
            - ppc: PowerPC architecture, only supported for AIX and Linux.
            - ppcle: PowerPC Little Endian architecture, only supported for Linux.
            - sparc: Sparc architecture, only supported for Solaris.
            - arm: ARM architecture, only supported for Linux.
            - s390: S/390 architecture, only supported for Linux. \n

        :returns AgentInstallerVersions: list of available versions of the OneAgent installer
        """
        params = {"flavor": flavor, "arch": arch}
        response = self.__http_client.make_request(path=f"{self.ENDPOINT_INSTALLER_AGENT}/versions/{os_type}/{installer_type}", params=params)

        return AgentInstallerVersions(raw_element=response.json())

    def get_gateway_installer_connection_info(self, network_zone: Optional[str] = "default") -> "ActiveGateConnectionInfo":
        """Gets the connectivity information for Environment ActiveGate.

        :param network_zone: The network zone you want the result to be configured with.

        :returns ActiveGateConnectionInfo: connectivity information
        """
        params = {"networkZone": network_zone}
        response = self.__http_client.make_request(path=f"{self.ENDPOINT_INSTALLER_GATEWAY}/connectioninfo", params=params)

        return ActiveGateConnectionInfo(raw_element=response.json())

    def list_gateway_installer_versions(self, os_type: str) -> "ActiveGateInstallerVersions":
        """Lists all available versions of ActiveGate installer.

        :param os_type: The operating system of the installer. Use one of:
            - windows
            - unix

        :returns ActiveGateInstallerVersions: all available versions of the installer
        """
        response = self.__http_client.make_request(path=f"{self.ENDPOINT_INSTALLER_GATEWAY}/versions/{os_type}")
        return ActiveGateInstallerVersions(raw_element=response.json())

    def get_gateway_installer(self, os_type: str, version: str = "latest", if_none_match: Optional[str] = None) -> "Response":
        """Downloads the configured standard ActiveGate installer.

        :param os_type: The operating system of the installer. Use one of:
            - windows
            - unix \n
        :param version: The required version of the ActiveGate installer, in 1.155.275.20181112-084458 format.
            If none is specified, latest available version is used.
        :param if_none_match: The ETag of the previous request. Do not download if it matches the ETag of the installer.
            The ETag is available in the headers of the response.

        :returns Response: HTTP Response to the request. Can be written to file from the "content" attribute.
        """
        if version != "latest":
            version = "version/" + version
        headers = {"If-None-Match": if_none_match} if if_none_match else None
        return self.__http_client.make_request(path=f"{self.ENDPOINT_INSTALLER_GATEWAY}/{os_type}/{version}", headers=headers)

    def list_boshrelease_agent_versions(self, os_type: str) -> "BoshReleaseAvailableVersions":
        """Lists available OneAgent versions for BOSH release tarballs.

        :param os_type: The operating system of the installer. Use one of:
            - windows
            - unix \n

        :returns BoshReleaseAvailableVersions: available versions
        """
        response = self.__http_client.make_request(path=f"{self.ENDPOINT_BOSHRELEASE}/versions/{os_type}")
        return BoshReleaseAvailableVersions(raw_element=response.json())

    def get_boshrelease_agent_checksum(
        self, os_type: str, version: str, skip_metadata: Optional[bool] = None, network_zone: Optional[str] = None
    ) -> "BoshReleaseChecksum":
        """Gets the checksum of the specified BOSH release tarball.
        The checksum is the sha256 hash of the installer file. For SaaS only works on environment ActiveGates version 1.176 or higher

        :param os_type: The operating system of the installer. Use one of:
            - windows
            - unix \n
        :param version: The required version of the OneAgent in the 1.155.275.20181112-084458 format.
        :param skip_metadata: Set true to omit the OneAgent connectivity information from the installer. If not set, false is used.
        :param network_zone: The network zone you want the result to be configured with.

        :returns BoshReleaseChecksum: checksum of the BOSH release tarball
        """
        params = {"skipMetadata": skip_metadata, "networkZone": network_zone}
        response = self.__http_client.make_request(path=f"{self.ENDPOINT_BOSHRELEASE}/agent/{os_type}/version/{version}/checksum", params=params)

        return BoshReleaseChecksum(raw_element=response.json())

    def get_boshrelease_agent(self, os_type: str, version: str, skip_metadata: Optional[bool] = None, network_zone: Optional[str] = None) -> "Response":
        """Downloads the BOSH release tarballs of the specified version, OneAgent included.
        For SaaS, the call is executed on an Environment ActiveGate. *Be sure to use the base URL of an ActiveGate, not the environment*

        :param os_type: The operating system of the installer. Use one of:
            - windows
            - unix \n
        :param version: The required version of the OneAgent in the 1.155.275.20181112-084458 format.
        :param skip_metadata: Set true to omit the OneAgent connectivity information from the installer. If not set, false is used.
        :param network_zone: The network zone you want the result to be configured with.

        :returns Response: HTTP Response to the request. Can be written to file from the "content" attribute.
        """
        params = {"skipMetadata": skip_metadata, "networkZone": network_zone}
        return self.__http_client.make_request(path=f"{self.ENDPOINT_BOSHRELEASE}/agent/{os_type}/version/{version}", params=params)

    def get_lambda_agent_versions(self) -> "LatestLambdaLayerNames":
        """Get the latest version names of the OneAgent for AWS Lambda.
        Version names include Java, Node.js, and Python AWS Lambda runtime.

        :returns LatestLambdaLayerNames: version names
        """
        return LatestLambdaLayerNames(raw_element=self.__http_client.make_request(path=f"{self.ENDPOINT_LAMBDA}").json())

    def get_orchestration_agent(self, orchestration_type: str, version: str = "latest") -> "Response":
        """Downloads the OneAgent deployment orchestration tarball.

        :param orchestration_type: The Orchestration Type of the orchestration deployment script. Use one of:
            - ansible
            - puppet \n
        :param version: The requested version of the OneAgent orchestration deployment tarball in 0.1.0.20200925-120822 format.
            If none is provided, the latest available is used.

        :returns Response: HTTP Response to the request. Can be written to file from the "content" attribute.
        """
        if version != "latest":
            version = "version/" + version
        return self.__http_client.make_request(path=f"{self.ENDPOINT_ORCHESTRATION}/{orchestration_type}/{version}")

    def get_orchestration_agent_signature(self, orchestration_type: str, version: str = "latest") -> "Response":
        """ ""Downloads the signature matching the OneAgent deployment orchestration tarball.

        :param orchestration_type: The Orchestration Type of the orchestration deployment script. Use one of:
            - ansible
            - puppet \n
        :param version: The requested version of the OneAgent orchestration deployment tarball in 0.1.0.20200925-120822 format.
            If none is provided, the latest available is used.

        :returns Response: HTTP Response to the request. Can be written to file from the "content" attribute.
        """
        if version != "latest":
            version = "version/" + version
        return self.__http_client.make_request(path=f"{self.ENDPOINT_ORCHESTRATION}/{orchestration_type}/{version}/signature")


class ConnectionInfo(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.tenant_uuid: str = raw_element["tenantUUID"]
        self.tenant_token: str = raw_element["tenantToken"]
        self.communication_endpoints: List[str] = raw_element.get("communicationEndpoints", [])
        self.formatted_communication_endpoints: str = raw_element["formattedCommunicationEndpoints"]


class InstallerMetaInfoDto(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.latest_agent_version: str = raw_element["latestAgentVersion"]


class AgentInstallerVersions(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.available_versions: List[str] = raw_element["availableVersions"]


class ActiveGateConnectionInfo(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.tenant_uuid: str = raw_element["tenantUUID"]
        self.tenant_token: str = raw_element["tenantToken"]
        self.communication_endpoints: str = raw_element["communicationEndpoints"]


class ActiveGateInstallerVersions(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.available_versions: List[str] = raw_element["availableVersions"]


class BoshReleaseChecksum(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.sha_256: str = raw_element["sha256"]


class BoshReleaseAvailableVersions(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.available_versions: List[str] = raw_element["availableVersions"]


class LatestLambdaLayerNames(DynatraceObject):
    def _create_from_raw_data(self, raw_element: Dict[str, Any]):
        self.java: str = raw_element["java"]
        self.python: str = raw_element["python"]
        self.nodejs: str = raw_element["nodejs"]
